<?php

/***********************************************************************************************************
* Subs-InlineAttachments.php - Subs of Post and PM Inline Attachments mod (v7.08)                          *
************************************************************************************************************
* This mod is licensed under the 2-Clause BSD License, which can be found here:                            *
*	https://opensource.org/licenses/BSD-2-Clause                                                           *
************************************************************************************************************
* CCopyright 2014-2020 Douglas Orend, 2018-2022 Kathy Leslie                                               *
* Redistribution and use in source and binary forms, with or without modification, are permitted provided  *
* that the following conditions are met:                                                                   *
* 1. Redistributions of source code must retain the above copyright notice, this list of conditions        *
*    and the following disclaimer.                                                                         *
* 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and *
*    the following disclaimer in the documentation and/or other materials provided with the distribution.  *
* This software is provided by the copyright holders and contributors "as is" and any express or implied   *
* warranties, including, but not limited to, the implied warranties of merchantability and fitness for a   *
* particular purpose are disclaimed. In no event shall the copyright holder or contributors be liable for  *
* any direct, indirect, incidental, special, exemplary, or consequential damages (including, but not       *
* limited to, procurement of substitute goods or services; loss of use, data, or profits; or business      *
* interruption) however caused and on any theory of liability, whether in contract, strict liability, or   *
* tort (including negligence or otherwise) arising in any way out of the use of this software, even if     *
* advised of the possibility of such damage.                                                               *
***********************************************************************************************************/

if (!defined('SMF'))
	die('Hacking attempt...');

//=======================================================================================
// BBCode hook functions & supporting sub-functions for mod.
//=======================================================================================
function ILA_Load_Theme()
{
	global $context, $modSettings;

	// Set default max width and height for inline attachments via CSS:
	$width = !empty($modSettings['ila_max_width']) ? $modSettings['ila_max_width'] . 'px' : '100%';
	$height = !empty($modSettings['ila_max_height']) ? $modSettings['ila_max_height'] . 'px' : '100%';

	$context['html_headers'] .= '
	<style>
		.ila_attach {margin: 0.5em 0 0.25em 0; width: auto; height: auto; max-width: ' . $width . '; max-height: ' . $height . ';}
		.ila_attach2 {margin-bottom: 0.25em;}
		.ila_link {margin-left: 0.4em; line-height: 1.4em; white-space: pre-wrap; overflow: hidden; text-overflow: ellipsis;}
		.ila_pad1 {padding-top: 0.25em; margin-left: 0;}
		.ila_pad2 {padding-top: 0.5em;}
		.ila_clip1 {float: left;}
		.ila_clip2 {float: left; margin-top: 0.25em;}
		.ila_below_post {display: inline-block; vertical-align: top; padding: 1em 1em 0 0 !important;}
		.ila_middletext {line-height: 1.4em;}
		.ila_table_cell {display: table-cell; vertical-align: top; padding-left: 0.4em; line-height: 1.4em;}
	</style>';
}

function ILA_tags()
{
	// The 'attachimg' tag is only included for backward compatibility for displaying inline attachments
	// that used this particular tag which was only included in some very early versions of the mod.
	// As such, it is not a valid tag option for inserting new images inline.
	return array('attach', 'attachment', 'attachmini', 'attachthumb', 'attachurl', 'attachimg');
}

function ILA_parameters(&$params1, &$params2)
{
	global $sourcedir;

	$params1 = array(
		'id' => array('match' => '(\d+)', 'validate' => 'ILA_Param_ID'),
		'width' => array('optional' => true, 'match' => '(\d+)', 'validate' => 'ILA_Param_Width'),
		'height' => array('optional' => true, 'match' => '(\d+)', 'validate' => 'ILA_Param_Height'),
		'float' => array('optional' => true, 'match' => '(left|right|center)', 'validate' => 'ILA_Param_Float'),
		'margin' => array('optional' => true, 'match' => '(\d+)', 'validate' => 'ILA_Param_Margin'),
		'margin-left' => array('optional' => true, 'match' => '(\d+)', 'validate' => 'ILA_Param_Margin_Left'),
		'margin-right' => array('optional' => true, 'match' => '(\d+)', 'validate' => 'ILA_Param_Margin_Right'),
		'margin-top' => array('optional' => true, 'match' => '(\d+)', 'validate' => 'ILA_Param_Margin_Top'),
		'margin-bottom' => array('optional' => true, 'match' => '(\d+)', 'validate' => 'ILA_Param_Margin_Bottom'),
	);
	$params2 = array_merge($params1, array(
		'border-style' => array('match' => '(none|dotted|dashed|solid|double|groove|ridge|inset|outset)', 'validate' => 'ILA_Param_Border_Style'),
		'border-width' => array('optional' => true, 'match' => '(\d+)', 'validate' => 'ILA_Param_Border_Width'),
		'border-color' => array('optional' => true, 'match' => '(#[\da-fA-F]{3}|#[\da-fA-F]{6}|[A-Za-z]{1,20}|rgb\(\d{1,3}, ?\d{1,3}, ?\d{1,3}\))', 'validate' => 'ILA_Param_Border_Color'),
	));
	$more = array(
		'scale' => array('optional' => true, 'match' => '(true|false|yes|no|invert)', 'validate' => 'ILA_Param_Scale'),
		'msg' => array('optional' => true, 'match' => '(new|\d+)', 'validate' => 'ILA_Param_Msg'),
	);
	$params1 = array_merge($params1, $more);
	$params2 = array_merge($params2, $more);
}

function ILA_BBCode(&$codes)
{
	ILA_parameters($params1, $params2);
	foreach (ILA_tags() as $tag)
	{
		// BBCode Usage: [attach=id,width,height]content ignored[/attach]
		$codes[] = array(
			'tag' => $tag,
			'type' => 'unparsed_commas_content',
			'test' => '(\d+|\d+,\d+|\d+,\d+,\d+|\d+,msg\d+|\d+,\d+,msg\d+|\d+,\d+,\d+,msg\d+)\]',
			'content' => '',
			'validate' => 'ILA_Start_v1x',
			'disabled_content' => '',
		);

		// BBCode Usage: [attach {params}]content ignored[/attach]
		$codes[] = array(
			'tag' => $tag,
			'type' => 'unparsed_content',
			'parameters' => $params2,
			'content' => '',
			'validate' => 'ILA_Start_v20',
			'disabled_content' => '',
		);
		$codes[] = array(
			'tag' => $tag,
			'type' => 'unparsed_content',
			'parameters' => $params1,
			'content' => '',
			'validate' => 'ILA_Start_v20',
			'disabled_content' => '',
		);
	}
}

//=======================================================================================
// Sub-function that deals with preparing for running the mod.
//=======================================================================================
function ILA_Load_Stuff()
{
	global $context, $boarddir;

	// Load the language strings for this mod, if available:
	if (file_exists($boarddir . '/Themes/default/languages/InlineAttachments.english.php'))
		loadLanguage('InlineAttachments');

	// Added language file for 'Who Downloaded Attachment' mod.
	if (file_exists($boarddir . '/Themes/default/languages/WhoDownloaded/WhoDownloaded.english.php'))
		loadLanguage('WhoDownloaded/WhoDownloaded');

	// Set some things up for performance benefits:
	$context['ila']['pm_attach'] = false;
	$context['ila_params'] = array();
	if (!isset($context['ila']['attachments']))
		$context['ila']['attachments'] = array();
	if (!isset($context['ila']['view_attachments']))
		$context['ila']['view_attachments'] = array();
	if (!isset($context['ila']['pm_view_attachments']))
		$context['ila']['pm_view_attachments'] = allowedTo('pm_view_attachments');
}

//=======================================================================================
// Sub-function gets attachment data so that inline attachments can be processed.
//=======================================================================================
function ILA_Setup($msg_id, &$message)
{
	global $context, $modSettings;

	// If there are news items the message ID will start with 'news' - skip these.
	if (substr($msg_id, 0, 4) == 'news')
		return;

	// Load language strings and stuff (duh)
	ILA_Load_Stuff();

	// We can't load attachments if we don't know the message id number....
	if (!empty($msg_id))
	{
		if (($id = (int) $msg_id) != 0)
			ILA_Post_Attachments($context['ila']['msg'] = $id);
		elseif (($id = (int) str_replace('pre', '', $msg_id)) != 0)
			ILA_Post_Attachments($context['ila']['msg'] = $id);
		elseif (function_exists('loadPMAttachmentContext'))
		{
			if (($id = (int) str_replace('pm', '', $msg_id)) != 0)
				ILA_PM_Attachments($context['ila']['msg'] = $id);
			elseif (($id = (int) str_replace('pm_pre', '', $msg_id)) != 0)
				ILA_PM_Attachments($context['ila']['msg'] = $id);
		}
	}

	if (empty($id))
		$id = ($msg_id == '' && isset($context['ila']['msg']) ? $context['ila']['msg'] : $msg_id);
	$context['ila_msg'] = $id;

	// Make damn sure that the "dont_show" element is defined for this message:
	if (!isset($context['ila']['dont_show'][$id]))
		$context['ila']['dont_show'][$id] = array();

	// Make damn sure that the "is_inline" element is defined for this message:
	if (!isset($context['ila']['is_inline'][$id]))
		$context['ila']['is_inline'][$id] = array();

	// If there isn't a message to setup for, just return to the caller:
	if (empty($message))
		return;

	// We need to screw up the ILA tags within the code tags so that they don't get "fixed":
	$pattern = '#\[code(?:.*?)?\](.+?)\[/code\]#i' . ($context['utf8'] ? 'u' : '');
	if (preg_match_all($pattern, $message, $codes, PREG_PATTERN_ORDER))
	{
		$temp_tag = '[' . md5(time());
		$codes = array_unique($codes[0]);
		foreach ($codes as $id => $code)
		{
			$temp_rep[$id] = str_replace('[attach', $temp_tag, $code);
			$message = str_replace($code, $temp_rep[$id], $message);
		}
	}

	// Convert v3.0+ tags into usable tags for the parser:
	$attach_id = !empty($modSettings['ila_one_based_numbering']) ? 1 : 0;
	foreach (ILA_tags() as $tag)
	{
		// Convert "[attach=n]" to "[attach=n][/attach]" and remove the stuff between the brackets:
		$pattern = '#\[' . $tag . '(=| )(.+?)\]([^\[]*)\[/' . $tag . '\]#i' . ($context['utf8'] ? 'u' : '');
		$message = preg_replace($pattern, '[' . $tag . '$1$2][/' . $tag . ']', $message);
		$pattern = '#\[' . $tag . '(=| )(.+?)\]#i' . ($context['utf8'] ? 'u' : '');
		$message = preg_replace($pattern, '[' . $tag . '$1$2][/' . $tag . ']', $message);
		$message = str_replace('[/' . $tag . '][/' . $tag . ']', '[/' . $tag . ']', $message);

		// Kludgey workaround for messages with autonumbering closed tags, courtsey of "dcmouser" @ SMF:
		$len = strlen( $findstr = '[' . $tag . ']' );
		do
		{
			$pos = strpos($message, $findstr);
			if ($pos !== false)
			{
				$message = substr_replace($message, '[' . $tag . '=' . $attach_id . '][/' . $tag . ']', $pos, $len);
				$attach_id++;
			}
		} while ($pos !== false);
	}

	// Let's fix the ILA tags within the code tags, since we screwed them up:
	if (isset($temp_rep))
	{
		foreach ($codes as $id => $code)
		{
			$temp_rep[$id] = str_replace('[attach', $temp_tag, $code);
			$message = str_replace($temp_rep[$id], $code, $message);
		}
	}

	// Process the inline attachments in the quotes, then pass the result back:
	ILA_Fix_Param_Order($message);
	ILA_Process_Quotes($message);
}

//=======================================================================================
// Sub-function dealing with gathering post attachments for ILA_Setup.
//=======================================================================================
function ILA_Post_Attachments($msg_id)
{
	global $context, $modSettings, $smcFunc, $attachments, $sourcedir;
	global $boarddir, $user_info;
	static $view_attachments = array();

	// Don't even attempt if attachments are disabled:
	$msg_id = (int) $msg_id;
	if (empty($msg_id) || empty($modSettings['attachmentEnable']))
		return;

	// Get the topic and board ID for the specified message:
	$request = $smcFunc['db_query']('', '
		SELECT id_member, id_board
		FROM {db_prefix}messages
		WHERE id_msg = {int:msg}',
		array(
			'msg' => $msg_id,
		)
	);
	$row = $smcFunc['db_fetch_assoc']($request);
	$smcFunc['db_free_result']($request);
	$context['ila']['id_member'][$msg_id] = isset($row['id_member']) ? $row['id_member'] : '0';
	$msg_board = (int) (isset($row['id_board']) ? $row['id_board'] : '0');

	// Check to make sure that we can view attachments for the board:
	if (!empty($msg_board) && !isset($view_attachments[$msg_board]))
	{
		$view_attachments[$msg_board] = allowedTo('view_attachments');
		if (!$view_attachments[$msg_board])
			$view_attachments[$msg_board] = allowedTo('view_attachments', $msg_board);
	}
	if (empty($view_attachments[$msg_board]))
		return;

	// Fetch attachments for use in "parse_bbc" function...
	$attachments[$msg_id] = array();
	$request = $smcFunc['db_query']('', '
		SELECT
			a.id_attach, a.id_folder, a.id_msg, a.filename, a.file_hash, COALESCE(a.size, 0) AS filesize,
			a.downloads, a.approved, a.width, a.height, a.*, COALESCE(thumb.id_attach, 0) AS id_thumb,
			thumb.width AS thumb_width, thumb.height AS thumb_height, thumb.id_folder AS thumb_folder,
			thumb.file_hash AS thumb_hash, thumb.filename AS thumb_name,
			m.id_topic, m.id_topic AS topic, m.id_board AS board
		FROM {db_prefix}attachments AS a
			LEFT JOIN {db_prefix}attachments AS thumb ON (thumb.id_attach = a.id_thumb)
			LEFT JOIN {db_prefix}messages AS m ON (m.id_msg = a.id_msg)
		WHERE a.id_msg = {int:message_id}
			AND a.attachment_type = {int:attachment_type}',
		array(
			'message_id' => $msg_id,
			'attachment_type' => 0,
			'is_approved' => 1,
		)
	);
	$temp = array();
	while ($row = $smcFunc['db_fetch_assoc']($request))
	{
		$temp[$row['id_attach']] = $row;
		if (!isset($attachments[$row['id_msg']]))
			$attachments[$row['id_msg']] = array();
	}
	$smcFunc['db_free_result']($request);

	// This is better than sorting it with the query...
	ksort($temp);
	foreach ($temp as $row)
		$attachments[$msg_id][] = $row;

	// Load the attachment context even if there are no attachments:
	$smf20 = (substr($modSettings['smfVersion'], 0, 3) == '2.0');
	if ($smf20)
	{
		// Is Tapatalk running? Include the correct copy of "Display.php"....
		if (!defined('IN_MOBIQUO'))
			require_once($sourcedir . '/Display.php');
		else
			require_once($boarddir . '/mobiquo/include/Display.php');

		// Load the attachment context even if there are no attachments:
		$context['ila']['attachments'][$msg_id] = loadAttachmentContext($msg_id, $user_info['id'], true);
	}
	else
	{
		require_once($sourcedir . '/Subs-Attachments.php');
		$context['ila']['attachments'][$msg_id] = loadAttachmentContext($msg_id, $attachments);
	}
}

//=======================================================================================
// Sub-function dealing with gathering PM attachments for ILA_Setup.
//=======================================================================================
function ILA_PM_Attachments($msg_id)
{
	global $context, $modSettings, $smcFunc, $attachments, $sourcedir, $user_info, $boarddir;

	// If attachments aren't enabled, why do anything?
	$msg_id = (int) $msg_id;
	if (empty($msg_id) || empty($modSettings['pmAttachmentEnable']) || empty($context['ila']['pm_view_attachments']))
		return;

	// Fetch attachments for use in "parse_bbc" function...
	unset($attachments[$msg_id]);
	$request = $smcFunc['db_query']('', '
		SELECT
			pa.id_attach, pa.id_folder, pa.id_pm, pa.pm_report, pa.filename, pa.file_hash,
			COALESCE(pa.size, 0) AS filesize, pa.downloads, pa.width, pa.height,
			COALESCE(thumb.id_attach, 0) AS id_thumb, thumb.width AS thumb_width, thumb.height AS thumb_height
		FROM {db_prefix}pm_attachments AS pa
			LEFT JOIN {db_prefix}pm_attachments AS thumb ON (thumb.id_attach = pa.id_thumb)
			LEFT JOIN {db_prefix}personal_messages AS pm ON (pm.id_pm = pa.id_pm)
			LEFT JOIN {db_prefix}pm_recipients AS pmr ON (pmr.id_pm = pa.id_pm AND pmr.id_member = {int:current_user})
		WHERE pa.attachment_type = {int:attachment_type}
			AND pa.id_pm = {int:msg_id}
			AND (pm.id_member_from = {int:current_user} OR pmr.id_member = {int:current_user})',
		array(
			'msg_id' => $msg_id,
			'attachment_type' => 0,
			'current_user' => $user_info['id'],
		)
	);
	$temp = array();
	while ($row = $smcFunc['db_fetch_assoc']($request))
	{
		$temp[$row['id_attach']] = $row;

		if (!isset($attachments[$row['id_pm']]))
			$attachments[$row['id_pm']] = array();
	}
	$smcFunc['db_free_result']($request);

	// This is better than sorting it with the query...
	ksort($temp);
	foreach ($temp as $row)
		$attachments[$row['id_pm']][] = $row;

	// Is Tapatalk running? Include the correct copy of "PersonalMessage.php"....
	if (!defined('IN_MOBIQUO'))
		require_once($sourcedir . '/PersonalMessage.php');
	else
		require_once($boarddir . '/mobiquo/include/PersonalMessage.php');

	// Load the attachment context even if there are no attachments:
	$context['ila']['attachments'][$msg_id] = loadPMAttachmentContext($msg_id);
	$context['ila']['pm_attach'] = true;
}

//=======================================================================================
// Functions that are called to alter ILA tags that have been quoted.
//=======================================================================================
function ILA_Process_Quotes(&$message, $msg_id = 0)
{
	global $context, $modSettings;

	$pattern = "#<\s*?form\b[^>]*>(.*?)</form\b[^>]*>#s";
	if (preg_match_all($pattern, $message, $quotes, PREG_PATTERN_ORDER))
	{
		$quotes = array_unique($quotes[0]);
		foreach ($quotes as $quote)
			$message = str_replace($quote, ILA_Add_MsgID($quote), $message);
	}
	if (!empty($msg_id))
		$message = ILA_Add_MsgID($message, $msg_id);
}

function ILA_Add_MsgID($message, $msg = 0)
{
	global $context;

	// Start searching for the SMF 2.0.x-style message ID in the quote bracket:
	if (empty($msg))
	{
		$pattern = '#\[quote(.+?)\#msg(\d+) (.+?)\]#i' . ($context['utf8'] ? 'u' : '');
		if (!preg_match_all($pattern, $message, $info, PREG_PATTERN_ORDER))
		{
			// Look for SMF 2.1-style message ID in the quote bracket:
			$pattern = '#\[quote(.+?)msg=(\d+) (.+?)\]#i' . ($context['utf8'] ? 'u' : '');
			if (!preg_match_all($pattern, $message, $info, PREG_PATTERN_ORDER))
				return ILA_Invalid_Tags($message, $msg, true);
		}
		$msg = $info[2][0];
	}
	foreach (ILA_tags() as $tag)
	{
		// Process the "Version 1.0" bbcode forms here:
		$pattern = '#\[' . $tag . '=(.+?)\]#i' . ($context['utf8'] ? 'u' : '');
		$message = preg_replace($pattern, '[' . $tag . '=$1,msg' . $msg . ']', $message);
		$pattern = '#msg(\d+),msg(\d+)#i' . ($context['utf8'] ? 'u' : '');
		$message = preg_replace($pattern, 'msg$1', $message);

		// Process the "Version 2.0" bbcode forms here:
		$pattern = '#\[' . $tag . ' (.+?)\]#i' . ($context['utf8'] ? 'u' : '');
		$message = preg_replace($pattern, '[' . $tag . ' $1 msg=' . $msg . ']', $message);
		$pattern = '#msg=(new|\d+) msg=(\d+)#i' . ($context['utf8'] ? 'u' : '');
		$message = preg_replace($pattern, 'msg=$1', $message);
	}
	return $message;
}

//=======================================================================================
// Function called to replace invalid attachment tags in the message.
//=======================================================================================
function ILA_Invalid_Tags($message, $msg_id = 0, $exclude_msgid = false)
{
	global $txt, $context, $modSettings, $topic;

	// Load language strings and stuff (duh)
	ILA_Load_Stuff();

	// Show attachment text string or error text string in topic history
	$replacement = (empty($modSettings['attachmentEnable']) || empty($context['ila']['view_attachments'][$topic]) ? $txt['ila_nopermission'] : $txt['ila_attachment']);
	foreach (ILA_tags() as $tag)
	{
		// Force each inline attachment tag into our mutant "Version 3.0" form :p
		$pattern = '#\[' . $tag . '(=| )(.+?)\]([^\[]*)\[/' . $tag . '\]#i' . ($context['utf8'] ? 'u' : '');
		$message = preg_replace($pattern, '[' . $tag . '$1$2]', $message);

		// If "$exclude_msgid" is false, clear all invalid inline attachments from the message:
		$pattern = '#\[' . $tag . '(=| )(.+?)\]#i' . ($context['utf8'] ? 'u' : '');
		if (empty($exclude_msgid))
			$message = preg_replace($pattern, $replacement, $message);

		// Otherwise, clear invalid inline attachments that don't have a message ID linked to it:
		elseif (preg_match_all($pattern, $message, $attachcode, PREG_PATTERN_ORDER))
		{
			$attachcode = array_unique($attachcode[0]);
			asort($attachcode);
			foreach ($attachcode as $code)
			{
				if (preg_match('# msg=(\d+)#i', $code, $msg_param))
				{
					if ($msg_param[1] == $msg_id)
						$message = str_replace($code, $replacement, $message);
				}
				elseif (preg_match('#,msg(\d+)#i', $code, $msg_param))
				{
					if ($msg_param[1] == $msg_id)
						$message = str_replace($code, $replacement, $message);
				}
				else
					$message = str_replace($code, $replacement, $message);
			}
		}
	}
	return $message;
}

//=======================================================================================
// Function to fix the ILA tags after an attachment has been removed.
//=======================================================================================
function ILA_Reorganize_Tags(&$message, &$query)
{
	global $context, $attachments, $modSettings;

	// Figure out where the attachments will be once the requested ones are deleted:
	$msg_id = $query['id_msg'];
	if (!isset($attachments[$msg_id]))
		ILA_Setup($msg_id, $msg_id);

	$i = intval(!empty($modSettings['ila_one_based_numbering']));
	$attach = array();
	foreach ($attachments[$msg_id] as $attachment)
	{
		if (is_array($query['not_id_attach']) && in_array($attachment['id_attach'], $query['not_id_attach']))
		{
			$attach[$attachment['id_attach']] = $i;
			$i += 1;
		}
	}

	// Convert all inline attachment tags to "Version 3.0" form:
	foreach (ILA_tags() as $tag)
	{
		// Force each inline attachment tag into our mutant "Version 3.0" form :p
		$pattern = '#\[' . $tag . '(=| )(.+?)\]([^\[]*)\[/' . $tag . '\]#i' . ($context['utf8'] ? 'u' : '');
		$message = preg_replace($pattern, '[' . $tag . '$1$2]', $message);

		// Start processing missing attachments:
		$pattern = '#\[' . $tag . '(=| )(.+?)\]#i' . ($context['utf8'] ? 'u' : '');
		if (preg_match_all($pattern, $message, $attachtags, PREG_PATTERN_ORDER))
		{
			// Find the unique attachment bbcodes and sort them so we don't change the same bbcode multiple times:
			$attachtags = array_unique($attachtags[0]);
			asort($attachtags);

			// Adjust or remove the attachment bbcodes so that the ones remaining still work:
			foreach ($attachtags as $txt)
			{
				if (preg_match('# id=(\d+)#i', $txt, $attach_num))
				{
					preg_match('# msg=(\d+)#i', $txt, $msg_num);
					if (isset($msg_num[1]) && $msg_num[1] == $msg_id)
					{
						$look_for = $attach_num[1] - intval(!empty($modSettings['ila_one_based_numbering']));
						if (!empty($attachments[$msg_id][$look_for]['id_attach']))
						{
							$id_attach = $attachments[$msg_id][$look_for]['id_attach'];
							if (!isset($attach[$id_attach]))
								$message = str_replace($txt, '', $message);
							else
							{
								$pattern = '#\[' . $tag . $attach_num[0] . '(.+?)\]#i' . ($context['utf8'] ? 'u' : '');
								$message = preg_replace($pattern, '[' . $tag . ' id=' . ((int) $attach[$id_attach]) . '$1]', $message);
							}
						}
						else
							$message = str_replace($txt, '', $message);
					}
				}
				elseif (preg_match('(\d+)', $txt, $attach_num))
				{
					if (!preg_match('#,msg(\d+)#i', $txt))
					{
						$look_for = $attach_num[0] - intval(!empty($modSettings['ila_one_based_numbering']));
						if (!empty($attachments[$msg_id][$look_for]['id_attach']))
						{
							$id_attach = $attachments[$msg_id][$look_for]['id_attach'];
							if (!isset($attach[$id_attach]))
								$message = str_replace($txt, '', $message);
							else
								$message = str_replace($txt, '[' . $tag . '=' . ((int) $attach[$id_attach]) . substr($txt, strlen($tag) + strlen($attach[$id_attach]) + 2), $message);
						}
						else
							$message = str_replace($txt, '', $message);
					}
				}
				else
					$message = str_replace($txt, '', $message);
			}
		}
	}
}

//=======================================================================================
// Function to fix the order of the parameters in v2 tags.
//=======================================================================================
function ILA_Fix_Param_Order(&$message)
{
	global $context;

	ILA_parameters($dummy, $parameters);
	foreach (ILA_tags() as $tag)
	{
		$pattern = '#\[' . $tag . ' (.+?)\]#i' . ($context['utf8'] ? 'u' : '');
		preg_match_all($pattern, $message, $matches);
		$matches = array_unique($matches[0]);
		asort($matches);
		foreach ($matches as $match)
		{
			$params = explode(' ', $match);
			unset($params[0]);
			$order = array();
			$replace_str = $old = $order[''] = '';
			foreach ($params as $param)
			{
				if (strpos($param, '=') === false && isset($order[$old]))
					$order[$old] .= ' ' . $param;
				else
					$order[$old = substr($param, 0, strpos($param, '='))] = substr($param, strpos($param, '=') + 1);
			}
			$out = '[' . $tag;
			foreach ($parameters as $key => $ignore)
				$out .= (isset($order[$key]) ? ' ' . $key . '=' . $order[$key] : '');
			$message = str_replace($match, $out, $message);
		}
	}
}

//=======================================================================================
// BBCode parameter validation functions.
//=======================================================================================
function ILA_Param_ID($id)
{
	global $context;
	$context['ila_params']['id'] = (int) $id;
}

function ILA_Param_Width($width)
{
	global $context;
	$context['ila_params']['width'] = max(0, (int) $width);
}

function ILA_Param_Height($height)
{
	global $context;
	$context['ila_params']['height'] = max(0, (int) $height);
}

function ILA_Param_Float($where)
{
	global $context;
	$context['ila_params']['float'] = (!empty($where) ? ($where == 'left' ? 'left' : ($where == 'right' ? 'right' : 'center')) : false);
}

function ILA_Param_Margin($margin)
{
	global $context;
	$context['ila_params']['margin'] = max(0, (int) $margin);
}

function ILA_Param_Margin_Left($margin)
{
	global $context;
	$context['ila_params']['margin-left'] = max(0, (int) $margin);
}

function ILA_Param_Margin_Right($margin)
{
	global $context;
	$context['ila_params']['margin-right'] = max(0, (int) $margin);
}

function ILA_Param_Margin_Top($margin)
{
	global $context;
	$context['ila_params']['margin-top'] = max(0, (int) $margin);
}

function ILA_Param_Margin_Bottom($margin)
{
	global $context;
	$context['ila_params']['margin-bottom'] = max(0, (int) $margin);
}

function ILA_Param_Border_Style($style)
{
	global $context;
	$context['ila_params']['border-style'] = $style;
}

function ILA_Param_Border_Width($width)
{
	global $context;
	$context['ila_params']['border-width'] = max(0, (int) $width);
}

function ILA_Param_Border_Color($color)
{
	global $context;
	$context['ila_params']['border-color'] = $color;
}

function ILA_Param_Scale($answer)
{
	global $context, $modSettings;
	if ($answer == 'invert')
		$context['ila_params']['scale'] = empty($modSettings['ila_enable_responsive']);
	else
		$context['ila_params']['scale'] = (!$answer || $answer == 'false' || $answer == 'no');
}

function ILA_Param_Msg($msg)
{
	global $context, $modSettings, $sourcedir;

	$context['ila_params']['msg'] = ($msg == 'new' ? 'new' : (int) $msg);
	$msg = (int) $msg;
	if (empty($modSettings['ila_allow_quoted_images']) || empty($msg))
		return;
	elseif (!isset($context['ila']['attachments'][$msg]) && empty($context['ila']['pm_attach']))
		ILA_Post_Attachments($msg, true);
	elseif (!isset($context['ila']['attachments'][$msg]) && !empty($context['ila']['pm_attach']))
		ILA_PM_Attachments($msg);
}

//=======================================================================================
// Validation functions.
//=======================================================================================
function ILA_Start_v1x(&$tag, &$data, &$disabled)
{
	global $context, $txt;

	if (!isset($data[1]))
		$tag['content'] = $txt['ila_invalid'];
	else
	{
		$context['ila_params'] = array(
			'id' => (int) $data[1],
			'width' => isset($data[2]) ? (int) $data[2] : 0,
			'height' => isset($data[3]) ? (int) $data[3] : 0,
		);
		if (substr($data[ count($data) - 1 ], 0, 3) == 'msg')
			ILA_Param_Msg( substr($data[ count($data) - 1 ], 3) );
		$tag['content'] = ILA_Build_HTML($tag, $context['ila_params']['id']);
	}
	unset($context['ila_params']);
}

function ILA_Start_v20(&$tag, &$data, &$disabled)
{
	global $context, $txt;

	if (!isset($context['ila_params']['id']))
		$tag['content'] = $txt['ila_invalid'];
	else
		$tag['content'] = ILA_Build_HTML($tag, $context['ila_params']['id']);
	unset($context['ila_params']);
}

//=======================================================================================
// Link building functions.
//=======================================================================================
function ILA_Build_HTML(&$tag, &$id)
{
	global $context, $modSettings, $settings, $txt, $sourcedir, $user_info, $smcFunc;

	// If the "one-based numbering" option is set, subtract 1 from the attachment ID to make it compatible:
	$id = $id - intval(!empty($modSettings['ila_one_based_numbering']));

	// Make sure that we can access other messages:
	$allowed = (isset($modSettings['ila_allow_quoted_images']) && !empty($modSettings['ila_allow_quoted_images']));
	if (!$allowed && (isset($context['ila']['msg']) && $context['ila_params']['msg'] != $context['ila']['msg']))
		return $txt['ila_noquoting'];
	if (isset($context['ila_params']['msg']))
		$msg = ($allowed || (isset($context['ila']['msg']) && $context['ila_params']['msg'] == $context['ila']['msg']) ? $context['ila_params']['msg'] : -1);
	else
		$msg = (isset($context['ila']['msg']) ? $context['ila']['msg'] : -1);

	// If we are previewing the post or PM, return "attachment not uploaded yet" message:
	if (((isset($_REQUEST['action']) ? $_REQUEST['action'] : '') == 'post2') && (!isset($context['ila']['attachments'][$msg][$id]))
	|| ((isset($_REQUEST['action']) ? $_REQUEST['action'] : '') == 'pm' && (isset($_REQUEST['sa']) ? $_REQUEST['sa'] : '') == 'send2'))
		return $txt['ila_not_uploaded'];

	// Added code to work with 'Images on Print Page' mod.
	if (isset($_REQUEST['action']) && $_REQUEST['action'] == 'printpage' && (empty($modSettings['attachmentEnable']) || !isset($context['ila']['attachments'][$msg])))
		return $txt['ila_printpage'];

	// Are attachments enabled and can we see them? If not, return no permission message:
	if ((empty($context['ila']['pm_attach']) && (empty($modSettings['attachmentEnable']) || !isset($context['ila']['attachments'][$msg])))
	|| (!empty($context['ila']['pm_attach']) && (empty($modSettings['pmAttachmentEnable']) || empty($context['ila']['pm_view_attachments']))))
		return $txt['ila_nopermission'];

	// Does the specified attachment exist in the message?
	// If not work out what should be shown ...
	if (!isset($context['ila']['attachments'][$msg][$id]))
	{
		if (!empty($modSettings['ila_no_invalid_attachment_warning']))
			return $txt['ila_invalid'];
		else
			return;
	}

	// Get the attachment information ...
	$attachment = &$context['ila']['attachments'][$msg][$id];

	// This part is done after board permission check because we don't want to give user false hope.
	// If we are previewing the post, return "attachment not uploaded yet" message:
	if (!isset($context['ila']['attachments'][$msg]) && (isset($_REQUEST['action']) ? $_REQUEST['action'] : '') == 'post2')
		return $txt['ila_not_uploaded'];

	// If the ILA tag format is '[attachment id=0 msg=new]' display the text ' [ Attachment ] ' ...
	if ($msg == 'new')
		return $txt['ila_attachment'];

	// Not sure if this actually ever gets used - leaving it here anyway ...
	// Does the specified attachment exist in the message? If not, return attachment invalid message:
	if (!isset($context['ila']['attachments'][$msg]))
		return $txt['ila_invalid'];

	// Return message that message is unapproved ONLY IF not post author or allowed to approve:
	if (empty($attachment['is_approved']) && (!empty($context['ila']['id_member'][$msg]) && $user_info['id'] != $context['ila']['id_member'][$msg] && !allowedTo('approve_posts')))
		return $txt['ila_unapproved'];

	// IMPORTANT: Set the download counter variable correctly!
	$download_count = ($tag['tag'] == 'attachurl' ? 4 : ($tag['tag'] == 'attachmini' ? 0 :
		(isset($modSettings['ila_download_count']) ? $modSettings['ila_download_count'] : 0)));

	// Start assembling the transparency CSS style:
	$style = min(100, max(0, (isset($modSettings['ila_transparent']) ? $modSettings['ila_transparent'] : 40)));
	if (empty($attachment['is_approved']) && empty($style))
		return $txt['ila_unapproved'];
	$style = (empty($attachment['is_approved']) && !$context['ila']['pm_attach'] ? 'opacity: ' . ($style / 100) . '; filter: alpha(opacity=' . $style . ');' : '');

	$dimensions = null;
	$no_download = (allowedTo('ila_no_download1') && !allowedTo('approve_posts') ? true : '');

	//=======================================================================================
	// Is this an image? If so, assemble the HTML necessary to show it.
	//=======================================================================================
	if (!empty($attachment['is_image']))
	{
		// Set the specified width and height in the CSS style:
		$width = !empty($context['ila_params']['width']) ? $context['ila_params']['width'] : 0;
		$height = !empty($context['ila_params']['height']) ? $context['ila_params']['height'] : 0;
		$style .= !empty($width) ? ' width: ' . $width . 'px;' : '';
		$style .= !empty($height) ? ' height: ' . $height . 'px;' : '';

		// Call the appropriate function to get the HTML we need:
		$has_thumb = !empty($attachment['thumbnail']['has_thumb']);
		if ($no_download)
			$function = 'ILA_tag_attachthumb';
		elseif ($tag['tag'] == 'attachmini' || $tag['tag'] == 'attachimg')
			$function = 'ILA_tag_attach';
		else
			$function = 'ILA_tag_' . (!empty($modSettings['ila_attach_same_as_attachment']) && $tag['tag'] == 'attach' ? 'attachment' : $tag['tag']);
		$html = $function($attachment, $dimensions, $has_thumb, !empty($style) ? 'style="' . $style . '"' : '');

		// If the option to show EXIF is checked, let's show the EXIF information (if available):
		if ($tag['tag'] != 'attachurl' && !empty($modSettings['ila_display_exif']) && isset($attachment['exif']))
			$html .= '<span class="smalltext">' . preg_replace('/\s+/', " ", $attachment['exif']) . '</span><br />' . (!empty($modSettings['ila_download_count']) && $tag['tag'] != 'attachmini' ? '' : '<br />');
	}
	//=======================================================================================
	// If this is not an image, see if we can still display the attachment.
	//=======================================================================================
	elseif (!$no_download)
	{
		$url = &$attachment['href'];
		$ext = strtolower(pathinfo($attachment['name'], PATHINFO_EXTENSION));

		// If we are allowed to embed attached SVG files, then do so:
		if (!empty($modSettings['ila_embed_svg_files']) && $ext == 'svg')
		{
			$width = (!empty($context['ila_params']['width']) ? $context['ila_params']['width'] : 0);
			$height = (!empty($context['ila_params']['height']) ? $context['ila_params']['height'] : 0);
			$dimensions = array('width' => $width, 'height' => $height);
			$html = '<img src="' . $url . '" alt="' . $attachment['name'] . '" class="bbc_img ila_attach lazyload"' . (empty($width) ? '' : ' width="' . $width . '"') . (empty($height) ? '' : ' height="' . $height . '"') . ' />';
		}
		// If we are allowed to embed attached TXT files, then do so:
		elseif (!empty($modSettings['ila_embed_txt_files']) && $ext == 'txt')
		{
			require_once($sourcedir . '/Subs-Package.php');
			$html = parse_bbc(fetch_web_data($url));
		}
		// If we are allowed to embed attached PDF files, then do so:
		elseif (!empty($modSettings['ila_embed_pdf_files']) && $ext == 'pdf')
		{
			$width = (isset($context['ila_params']['width']) ? $context['ila_params']['width'] : 500);
			$height = (isset($context['ila_params']['height']) ? $context['ila_params']['height'] : 600);
			$html =
				'<object class="ila_attach" data="' . $url . '" type="application/pdf" width="' . $width . '" height="' . $height . '">
					<iframe src="' . $url . '" style="border: none;" width="' . $width . '" height="' . $height . '">
						' . $txt['ila_pdf1'] .
						' <a href="' . $url . '">' . $txt['ila_pdf2'] . '</a>' .
					'</iframe>' .
				'</object>';
		}
		// If we are allowed to embed videos, then do so for supported formats:
		elseif (!empty($modSettings['ila_embed_video_files']) && in_array($ext, array('avi', 'wmv', 'mp4', 'ogv', 'webm')))
		{
			$width = (!empty($context['ila_params']['width']) ? $context['ila_params']['width'] :
				(!empty($modSettings['ila_video_default_width']) ? $modSettings['ila_video_default_width'] : 640));
			$height = (!empty($context['ila_params']['height']) ? $context['ila_params']['height'] :
				(!empty($modSettings['ila_video_default_height']) ? $modSettings['ila_video_default_height'] : 400));
			$dim = ' width="' . $width . '" height="' . $height . '"';
			$show_video_link = isset($modSettings['ila_video_show_download_link']) ? $modSettings['ila_video_show_download_link'] : 0;
			$download_count = ($show_video_link == 1 && $download_count == 0 ? $download_count = 1 : ($show_video_link == 1 && $download_count != 0 ? $download_count : 0));

			if ($ext == 'avi')
				$html =
					'<object classid="clsid:67DABFBF-D0AB-41fa-9C46-CC0F21721616"' . $dim . ' codebase="https://go.divx.com/plugin/DivXBrowserPlugin.cab">' .
						'<param name="mode" value="full" />' .
						'<param name="autoPlay" value="false" />' .
						'<param name="src" value="' . $url . '" />' .
						'<embed type="video/divx" src="' . $url . '" mode="full"' . $dim . ' autoPlay="false" pluginspage="https://go.divx.com/plugin/download/" />' .
					'</object>';
			elseif ($ext == 'wmv')
				$html =
					'<object classid="clsid:22D6F312-B0F6-11D0-94AB-0080C74C7E95"' . $dim . ' standby="Loading Windows Media Player components..." type="application/x-oleobject" id="MediaPlayer">' .
						'<param name="FileName" value="' . $url . '">' .
						'<param name="autostart" value="false">' .
						'<param name="ShowControls" value="true">' .
						'<param name="ShowStatusBar" value="false">' .
						'<param name="ShowDisplay" value="false">' .
						'<embed type="application/x-mplayer2" src="' . $url . '" name="MediaPlayer"' . $dim . ' ShowControls="1" ShowStatusBar="0" ShowDisplay="0" autostart="0"></embed>' .
					'</object>';
			elseif ($ext == 'mp4' || $ext == 'ogv' || $ext == 'webm')
			{
				// Search for other video attachments with similar names:
				$file = array();
				$filename = pathinfo($attachment['name'], PATHINFO_FILENAME);
				foreach (array('mp4', 'ogv', 'webm', 'jpg', 'png') as $ext)
				{
					if ($attachment['name'] == $filename . '.' . $ext)
						$file[$ext] = $attachment;
				}

				// Build the video HTML to show the user:
				$img = (!empty($file['png']) ? $file['png']['href'] : (!empty($file['jpg']) ? $file['jpg']['href'] : ''));
				$html5 = !empty($modSettings['ila_video_html5']);
				$html = ($html5 ?
					'<video class="ila_attach" controls="controls" width="' . $width . '" height="' . $height . '">' .
						(!empty($file['mp4']['href']) ? '<source src="' . $file['mp4']['href'] . '" type="video/mp4" />' : '') .
						(!empty($file['ogv']['href']) ? '<source src="' . $file['ogv']['href'] . '" type="video/ogv" />' : '') .
						(!empty($file['webm']['href']) ? '<source src="' . $file['webm']['href'] . '" type="video/webm" />' : '') : '') .
					($html5 && (!empty($file['mp4']['href']) || !empty($file['webm']['href'])) ?
						'<object type="application/x-shockwave-flash" width="' . $width . '" height="' . $height . '">' .
							'<param name="movie" />' .
							'<param name="allowFullScreen" value="true" />' .
							'<param name="wmode" value="transparent" />' .
							'<param name="flashVars" value="controlbar=over&amp;' . (!empty($img) ? 'image=' . urlencode($img) . '&amp;' : '') . 'file=' . (!empty($file['mp4']['href']) ? urlencode($file['mp4']['href']) : urlencode($file['webm']['href'])) . '" />' .
							(!empty($img) ? '<img src="' . $img . '" width="' . $width . '" height="' . $height . '" title="' . $txt['ila_no_video'] . '" />' : '<br /><strong>' . $txt['ila_no_video']) . '</strong>' .
						'</object>' : '') .
					($html5 ? '</video>' : '');
			}
		}
	}

	//=======================================================================================
	// Add the download counters if requested.
	//=======================================================================================
	if (empty($html) || $download_count || $no_download)
		$html = (!empty($html) ? $html . '<br />' : '') . ILA_download_counter($download_count, $attachment, $dimensions, $no_download);

	// Do we have something to float or put a margin around?
	if (!empty($html))
	{
		// Process all the margin parameters:
		$style = array();
		if (isset($context['ila_params']['margin']))
			$style[] = 'margin: ' . $context['ila_params']['margin'] . 'px;';
		if (isset($context['ila_params']['margin-left']))
			$style[] = 'margin-left: ' . $context['ila_params']['margin-left'] . 'px;';
		if (isset($context['ila_params']['margin-right']))
			$style[] = 'margin-right: ' . $context['ila_params']['margin-right'] . 'px;';
		if (isset($context['ila_params']['margin-top']))
			$style[] = 'margin-top: ' . $context['ila_params']['margin-top'] . 'px;';
		if (isset($context['ila_params']['margin-bottom']))
			$style[] = 'margin-bottom: ' . $context['ila_params']['margin-bottom'] . 'px;';
		if (isset($context['ila_params']['border-style']))
			$style[] = 'border-style: ' . $context['ila_params']['border-style'] . ';';
		if (isset($context['ila_params']['border-width']))
			$style[] = 'border-width: ' . $context['ila_params']['border-width'] . 'px;';
		if (isset($context['ila_params']['border-color']))
			$style[] = 'border-color: ' . $context['ila_params']['border-color'] . ';';
		$style = implode(' ', $style);

		// Add the margin and float params to the rest of the HTML:
		if (isset($context['ila_params']['float']) && $context['ila_params']['float'] == 'center')
			$html = '<div style="margin-left: auto; margin-right: auto; display: block;">' . $html . '</div>';
		elseif (isset($context['ila_params']['float']))
			$html = '<div style="display: inline-block; float: ' . $context['ila_params']['float'] . ';' . (!empty($style) ? $style : '') . '">' . $html . '</div>';
		elseif (!empty($style))
		{
			if ((!empty($modSettings['ila_download_count']) && $tag['tag'] != 'attachmini') || $tag['tag'] == 'attachurl')
				$html = '<div style="display: inline-block; ' . (!empty($style) ? $style : '') . '">' . $html . '</div>';
			else
				$html = str_replace('<img src="', '<img style="' . $style . '" src="', $html);
		}
		else
			$html = '<div style="display: inline-block; padding-right: 0.5em;">' . $html . '</div>';
	}

	// For approved attachments ...
	if (!empty($attachment['is_approved']))
	{
		if (!empty($modSettings['ila_duplicate']))
			// Mark as "don't show" if admin has checked that option ...
			$context['ila']['dont_show'][$msg][$attachment['id']] = true;
		else
			// This attachment is being displayed inline ...
			$context['ila']['is_inline'][$msg][$attachment['id']] = true;

		// For SMF 2.1: This attachment should not be displayed below the post ...
		$smf21 = (substr($modSettings['smfVersion'], 0, 3) == '2.1');
		if ($smf21 && !empty($modSettings['dont_show_attach_under_post']) && !isset($context['show_attach_under_post'][$attachment['id']]))
			$context['show_attach_under_post'][$attachment['id']] = $attachment['id'];
	}

	// Check option for PMs ...
	if (!empty($modSettings['ila_duplicate']) && $context['ila']['pm_attach'])
		$context['ila']['dont_show'][$attachment['id']] = true;

	// Return to our lord and saviour, our caller! :p
	return $html;
}

//=======================================================================================
// Helper functions that return HTML on how to display each attachment inline.
//=======================================================================================
function ILA_subfunction($id, $full, $thumb, $name, $id_thumb = '', $style = '', $has_thumb = false, $expand = true)
{
	global $sourcedir, $settings, $modSettings, $context, $msg, $user_info, $txt;

	// Yup, you read right: Increase attachment ID by one million! 99.99+% chance
	// of no conflict while showing attachments as thumbnails below post!
	$id += 1000000;
	if (!empty($id_thumb))
		$id_thumb += 1000000;

	// Added code to work with 'Images on Print Page' mod.
	if (isset($_REQUEST['action']) && $_REQUEST['action'] == 'printpage')
		return '<img src="' . ($user_info['id'] == 0 || !empty($modSettings['ila_ipp_thumbnail']) ? $thumb : $full) . '" alt="" class="ila_attach" />';

	// Are we asked not to scale the image in ANY WAY?
	$class = !empty($modSettings['ila_enable_responsive']) && empty($context['ila_params']['scale']) ? 'ila_attach lazyload' : 'lazyload';

	// Only show a thumbnail if the full size image cannot be downloaded.
	$no_download = (allowedTo('ila_no_download1') && !allowedTo('approve_posts') ? true : '');
	if ($no_download)
		return '<a title="' . $name . '" onclick="alert(\'' . $txt['ila_login_view_img'] . '\'); return false;"><img src="' . $thumb . '" alt="" ' . (!empty($class) ? 'class="' . $class . '" ' : '') . '/></a>';

	// Is the image expandable using known highslide/lightbox viewers?
	if ($expand && !empty($modSettings['ila_highslide']))
	{
		// Load any other necessary files: (NOTE: Provided as courtesy to dcmouser)
		if (!empty($modSettings['dc_mod_enable_highslideviewer']) && file_exists($sourcedir . '/DcSubsHighslideImageViewer.php'))
			require_once($sourcedir . '/DcSubsHighslideImageViewer.php');

		// HS4SMF Installed?
		if (!empty($modSettings['hs4smf_enabled']) && function_exists('hs4smf_get_slidegroup'))
		{
			$context['hs4smf_img_count'] = !empty($context['hs4smf_img_count']) ? $context['hs4smf_img_count'] + 1 : 1;
			$msgid = (int) $msg;
			hs4smf_track_slidegroup($msgid); // is this needed?
			$slidegroup = hs4smf_get_slidegroup($msgid);
			return '<a href="' . $full . ';image" id="link_' . $id . '" class="highslide' . (!empty($class) ? ' ' . $class : '') . '" onclick="return hs.expand(this, ' . $slidegroup . ')"><img src="' . $thumb . '" alt="' . $name . '"' . ' id="thumb_' . $id_thumb . '"' . $style . (!empty($class) ? ' class="' . $class . '"' : '') . ' /></a>';
		}
		// Highslide Image Viewer Installed?
		elseif (function_exists('highslide_images'))
			return '<a href="' . $full . ';image" id="link_' . $id . '" class="highslide' . (!empty($class) ? ' ' . $class : '') . '" rel="highslide"><img src="' . $thumb . '" alt="' . $name . '"' . ' id="thumb_' . $id_thumb . '"' . $style . (!empty($class) ? ' class="' . $class . '"' : '') . ' /></a>' . (isset($context['subject']) ? '<span class="highslide-heading">' . $context['subject'] . '</span>' : '');
		// jQLightbox Installed?
		elseif (!empty($modSettings['enable_jqlightbox_mod']) && strpos($context['html_headers'], 'jquery.prettyPhoto.css'))
			return '<a href="' . $full . ';image" id="link_' . $id . '" rel="lightbox[gallery]" ' . (!empty($class) ? ' class="' . $class . '"' : '') . '><img src="' . $thumb . '" alt="' . $name . '"' . ' id="thumb_' . $id_thumb . '"' . (!empty($class) ? ' class="' . $class . '"' : '') . $style . ' /></a>';
	}

	// Okay, can't show via known highslide/lightbox viewers. Show it via SMF methods:
	if ($has_thumb)
		return ($id == $id_thumb ? '<a href="' . $full . '" id="link_' . $id . '" onclick="return expandThumb(' . $id . ');">' : '') . '<img src="' . $thumb . '" alt="" id="thumb_' . $id_thumb . '"' . (!empty($class) ? ' class="' . $class . '"' : '') . $style . ' />' . ($id == $id_thumb ? '</a>' : '');
	else
		return '<img src="' . $full . '" alt="' . $name . '" class="bbc_img resized ' . (!empty($class) ? $class : '') . '"' . $style . ' />';
}

// ILA tag 'attachment' => Show full size image.
function ILA_tag_attachment(&$info, &$dim, $has_thumb, $style)
{
	$dim = array('width' => $info['real_width'], 'height' => $info['real_height']);
	$thumbID = (!empty($info['thumbnail']['id']) ? $info['thumbnail']['id'] : '');
	return ILA_subfunction($info['id'], $info['href'], $info['href'], $info['name'], $thumbID, $style);
}

// ILA tag 'attach', 'attachmini', or 'attachimg' => Show thumbnail, expandable to full image.
function ILA_tag_attach(&$info, &$dim, $has_thumb, $style)
{
	$dim = array('width' => $info['real_width'], 'height' => $info['real_height']);
	$thumbID = $info['id'];
	// When attachment images are quoted from another post the topic number in $info['thumbnail']['href']
	// needs to be changed to the same topic number the image is actually attached to.
	if (isset($info['thumbnail']['href']) && $info['thumbnail']['href'] != $info['href'])
		$info['thumbnail']['href'] = substr($info['href'], 0,strrpos($info['href'], ';attach=') + 8) . $info['thumbnail']['id'] . ';image';
	$image = ($expand = !empty($info['thumbnail']['has_thumb'])) ? $info['thumbnail']['href'] : $info['href'];
	return ILA_subfunction($info['id'], $info['href'], $image, $info['name'], $thumbID, $style, $has_thumb, $expand);
}

// ILA tag 'attachthumb' => Show thumbnail ONLY, not expandable.
function ILA_tag_attachthumb(&$info, &$dim, $has_thumb, $style)
{
	$data = &$info;
	if (!empty($info['thumbnail']['has_thumb']))
		$dim = array('width' => $info['width'], 'height' => $info['height'], 'img' => $info['href']);
	else
		$dim = array('width' => $info['real_width'], 'height' => $info['real_height']);
	$thumbID = (!empty($info['thumbnail']['id']) ? $info['thumbnail']['id'] : '');
	// When attachment images are quoted from another post the topic number in $info['thumbnail']['href']
	// needs to be changed to the same topic number the image is actually attached to.
	if (isset($info['thumbnail']['href']) && $info['thumbnail']['href'] != $info['href'])
		$info['thumbnail']['href'] = substr($info['href'], 0,strrpos($info['href'], ';attach=') + 8) . $info['thumbnail']['id'] . ';image';
	$image = !empty($info['thumbnail']['has_thumb']) ? $info['thumbnail']['href'] : $info['href'];
	return ILA_subfunction($info['id'], $info['href'], $image, $info['name'], $thumbID, $style, $has_thumb);
}

// ILA tag 'attachurl' => Shows attachment size, image dimensions, and download count; no image.
function ILA_tag_attachurl(&$info, &$dim, $has_thumb, $style)
{
	$dim = array('width' => $info['real_width'], 'height' => $info['real_height']);
	return false;
}

//=======================================================================================
// Helper function to return the correct MIME type for certain types of files.
//=======================================================================================
function ILA_mime_type($filename, $ext, $original = false)
{
	// Set up for audio/video detection:
	$mime = false;
	$ext = pathinfo($original, PATHINFO_EXTENSION);
	$signatures = array(
	// Audio file signatures:
		/* wav */ "0|\x52\x49\x46\x46" => 'audio/wav|8|' . "\x57\x41\x56\x45",
		/* mp3 */ "0|\xFF\xFB" => 'audio/mpeg',
		/* mp3 */ "0|\x49\x44\x33" => 'audio/mpeg',
		/* m4a */ "4|\x66\x74\x79\x70\x4D\x53\x4E\x56" => 'audio/mp4',
		/* aac */ "0|\xFF\xF1" => 'audio/aac',
		/* aac */ "0|\xFF\xF9" => 'audio/aac',
		/* aac */ "0|\xFF\xFE" => 'audio/aac',
	// Video file signatures:
		/* mp4 */ "4|\x66\x74\x79\x70\x69\x73\x6F\x6D" => 'video/mp4',
		/* m4v */ "4|\x66\x74\x79\x70\x6D\x70\x34\x32" => 'video/mp4',
		/* webm */ "0|\x1A\x45\xDF\xA3" => 'video/webm',
	// Audio/Video file signature (could be either):
		/* ogg */ "0|\x4F\x67\x67\x53" => 'audio/ogg',
		/* wma/v */ "0|\x30\x26\xB2\x75\x8E\x66\xCF\x11" => 'audio/wma',
	// ALWAYS LAST CASE! Must return "FALSE" if we get here!
		/* N/A */ "0|" => false,
	);

	// Start checking against known signatures:
	if ($handle = @fopen($filename, 'rb'))
	{
		$contents = @fread($handle, 64);
		@fclose($handle);
		foreach ($signatures as $id => $mime_type)
		{
			list($start1, $magic_bytes) = explode('|', $id, 2);
			list($mime, $start2, $extra) = explode('|', $mime_type . '||');
			if (substr($contents, intval($start1), strlen($magic_bytes)) == $magic_bytes)
			{
				if (empty($mime) || substr($contents, intval($start2), strlen($extra)) == $extra)
					break;
			}
		}
	}

	// Since a file few signatures appear in both video and audio formats, we need to
	// look at the file extension to determine which mime type to return:
	if ($mime == 'audio/ogg')
		return $ext == 'ogv' ? 'video/ogg' : $mime;
	elseif ($mime == 'audio/wma')
		return $ext == 'wmv' ? 'video/wmv' : $mime;
	elseif ($ext == 'svg')
		return 'image/svg+xml';
	elseif (!empty($mime))
		return $mime;
	elseif (function_exists('finfo_file') && function_exists('finfo_open') && defined('FILEINFO_MIME_TYPE'))
		return finfo_file(finfo_open(FILEINFO_MIME_TYPE), $filename);
	elseif (function_exists('mime_content_type'))
		return mime_content_type($filename);
	else
		return false;
}

//=======================================================================================
// Function to return download counter.
//=======================================================================================
function ILA_download_counter($download_count, $attachment, $dimensions = null, $no_download = false, $below_post = false)
{
	global $context, $modSettings, $scripturl, $settings, $sourcedir, $txt, $user_info;

	// Added code to work with 'Images on Print Page' mod.
	if (isset($_REQUEST['action']) && $_REQUEST['action'] == 'printpage')
		return;

	$below_counter = (!empty($modSettings['ila_below_post_count']) ? $modSettings['ila_below_post_count'] : $download_count);

	// Prepare certain elements so that building the HTML code looks a little nicer:
	$ila_filesize = $txt['ila_filesize'] . $attachment['size'];

	// Dimensions only shown for images.
	if ($attachment['is_image'])
		$ila_dimensions = $txt['ila_dimensions'] . $dimensions['width'] . 'x' . $dimensions['height'];
	else
		$ila_dimensions = '';

	$ila_downloads = ($attachment['is_image'] ? $txt['ila_views'] : $txt['ila_downloads']) . $attachment['downloads'];

	// Who Downloaded Attachment mod integration for non-image attachments.
	if (file_exists($sourcedir . '/Mod-WhoDownloadedAttachment.php') && !$attachment['is_image'])
		$show_download_list = (allowedTo('show_download_list') ? '[<a href="javascript:void(0)" onclick="showWhoDownloadedAttachmentList(' . $attachment['id'] . ')">' . $txt['attachment_download_list'] . '</a>]<span id="download_list_' . $attachment['id'] . '"></span>' : '');
	else
		$show_download_list = '';

	// Download counter settings ...
	// 0 = Disabled
	// 1 = Filename only
	// 2 = Filename and Size
	// 3 = Filename, Size and Dimensions
	// 4 = Filename, Size, Dimensions and Count
	// 5 = Filename, Size, Dimensions and Count (2 lines)
	// 6 = Filename, Size, Dimensions and Count (3 lines)
	// 7 = Filename, Size and Dimensions (2 lines)
	// 8 = Filename, Size and Dimensions (3 lines)
	// 9 = Filesize and Count - non-image files (eg, mp3, pdf, zip, etc) only (2 lines)

	// Workout which download_count setting to use.
	$download_count = ($below_post ? (!$attachment['is_image'] && !$below_counter ? 1 : $below_counter) : (!$attachment['is_image'] && !$download_count ? 1 : $download_count));

	if ((!empty($download_count) && $download_count != 9) || ($download_count == 9 && !$attachment['is_image']))
	{
		$clipPNG = $settings['images_url'] . '/icons/clip.png';
		$clipGIF = $settings['images_url'] . '/ila_clip.gif';
		$clipIcon = (file_exists($clipPNG) ? $clipPNG : $clipGIF);

		// Let's build the HTML code for the download count options ...
		$ila_download_counter =
			'<span class="' . (!$below_post ? 'smalltext ila_pad1' : '') . '" style="display: ' . ($download_count <= 4 ? 'inline-block' : 'inline') . ';">' .
				'<span style="display: table;"><img class="' . ($no_download ? 'ila_clip1' : 'ila_clip2') . '" src="' . $clipIcon . '" align="middle" alt="*" /> ' .
				'<span class="ila_table_cell"><a class="ila_link ' . ($download_count >= 5 ? 'ila_pad1' : 'ila_pad2') . '" ' . ($no_download ? 'onclick="alert(\'' . $txt['ila_login_download'] . '\'); return false;"' : 'href="' . $attachment['href'] . '"') . ' title="' . $attachment['name'] . '" ><strong>' . $attachment['name'] . '</strong></a> ' . ($no_download ? '' :
				($download_count >= 5 ? '<br />' : '') .
				($download_count == 2 || $download_count == 6 || $download_count == 8 ? '(' . $ila_filesize . ')' : '') .
				($download_count == 3 || $download_count == 7 ? '(' . $ila_filesize . ($attachment['is_image'] ? ', ' . $ila_dimensions . ')' : ')') : '') .
				($download_count == 4 || $download_count == 5 || $download_count == 9 ? '(' . $ila_filesize . ', ' . ($attachment['is_image'] ? $ila_dimensions . ', ' : '') . $ila_downloads . ')' : '') .
				($download_count == 6 ? '<br />(' . ($attachment['is_image'] ? $ila_dimensions . ', ' : '') . $ila_downloads . ')' : '') .
				($download_count == 8 && $attachment['is_image'] ? '<br />(' . $ila_dimensions . ')' : '') .
				($show_download_list ? '<br /><span>' . $show_download_list . '</span>' : '') . '</span></span>') .
			'</span>';
	}
	else
		$ila_download_counter = '';

	return $ila_download_counter;
}

//=======================================================================================
// Function for displaying images below the post.
//=======================================================================================
function ILA_Below_Post_Show(&$message)
{
	global $context, $modSettings, $scripturl, $settings, $sourcedir, $txt, $user_info;

	if (empty($message['attachment']))
		return;

	$msg_id = $message['id'];
	$count = 0;
	$row_below = !empty($modSettings['ila_below_post_show']);
	$per_row = ($row_below ? (empty($modSettings['ila_below_post_number']) ? 0 : $modSettings['ila_below_post_number']) : 1);
	$download_inline = allowedTo('ila_no_download1');
	$download_below = allowedTo('ila_no_download2');
	$highslide = (file_exists($settings['default_theme_dir'] . '/Highslide/highslide.css') ? true : '');
	$last_approved_state = 1;

	echo '
								<div class="attachments smalltext" style="padding-top: 0;">';

	foreach ($message['attachment'] as $attachment)
	{
		$dont_show = (!empty($context['ila']['dont_show'][$msg_id][$attachment['id']]) ? $context['ila']['dont_show'][$msg_id][$attachment['id']] : '');
		$hide_unapproved = (!$attachment['is_approved'] && $message['member']['id'] != $context['user']['id'] && !$context['can_approve'] ? true : '');

		// Skip over already-displayed inline attachments and unapproved attachments that should be hidden.
		if ($dont_show || $hide_unapproved)
			continue;

		$download_count = 0;
		$is_inline = (!empty($context['ila']['is_inline'][$msg_id][$attachment['id']]) ? $context['ila']['is_inline'][$msg_id][$attachment['id']] : '');
		$no_download = (!allowedTo('approve_posts') && (($download_inline && $download_below) || ($is_inline && $download_inline) || (!$is_inline && $download_below)) ? true : '');
		$dimensions = ($attachment['is_image'] ? $dimensions = array('width' => $attachment['real_width'], 'height' => $attachment['real_height']) : '');
		$download_counter = ILA_download_counter($download_count, $attachment, $dimensions, $no_download, true);
		$per_row = (!$attachment['is_approved'] ? 1 : $per_row);

		// Show a special box for unapproved attachments ...
		if ($attachment['is_approved'] != $last_approved_state)
		{
			$last_approved_state = 0;

			echo '
									<fieldset style="margin-top: 1.0em;">
										<legend>&nbsp;&nbsp;', $txt['attach_awaiting_approve'];

			if ($context['can_approve'])
				echo '
											&nbsp;[<a href="', $scripturl, '?action=attachapprove;sa=all;mid=', $message['id'], ';', $context['session_var'], '=', $context['session_id'], '">', $txt['approve_all'], '</a>]';

			echo '
										&nbsp;</legend>
										<div style="padding-bottom: 0.5em;">';
		}
		else
			echo '
										<div class="ila_below_post">';

		if (file_exists($sourcedir . '/Subs-MediaAttachments.php') && !$no_download)
		{
			if (!empty($attachment['is_audio']))
			{
				$width = max((!empty($modSettings['attachmentAudioPlayerWidth']) ? ($modSettings['attachmentAudioPlayerWidth'] / 2) : '240'), 240);
				echo '
										<audio controls="controls" class="ila_attach2" style="width: ' . $width . 'px;">
											<source src="', $attachment['href'], '" type="', $attachment['mime_type'], '">
											<embed src="', $attachment['href'], '" width="' . $width . '" height="80" loop="false" autostart="false">
											', $txt['PMA_no_audio'], '
										</audio><br />';
			}
			if (!empty($attachment['is_video']))
			{
				$width = max((!empty($modSettings['attachmentVideoPlayerWidth']) ? ($modSettings['attachmentVideoPlayerWidth'] / 2) : '320'), 320);
				echo '
										<video controls="controls" class="ila_attach2" style=" width: ' . $width .'px;">
											<source src="', $attachment['href'], '"' . ($attachment['mime_type'] != 'video/quicktime' ? ' type="' . $attachment['mime_type'] . '"' : '') . '>
											<embed src="', $attachment['href'], '" width="' . $width . '" loop="false" autostart="false">
											', $txt['PMA_no_video'], '
										</video><br />';
			}
		}

		if ($attachment['is_image'])
		{
			$has_thumb = $attachment['thumbnail']['has_thumb'];
			if ($no_download)
				echo '
											<a title="' . $attachment['name'], '" onclick="alert(\'' . $txt['ila_login_view_img'] . '\'); return false;"><img src="' . ($has_thumb ? $attachment['thumbnail']['href'] : $attachment['href']) . '" alt="" /></a><br />';
			elseif ($has_thumb)
				echo '
											<a href="', $attachment['href'], '" id="link_', $attachment['id'], '" class="highslide" rel="highslide' . (!$highslide ? ' new_win nofollow" target="_blank' : '') . '"><img class="ila_attach lazyload" src="', $attachment['thumbnail']['href'], '" alt="', $attachment['name'], '" id="thumb_', $attachment['id'], '" border="0" /></a><br />' . ($highslide ? '<span class="highslide-heading">' . $context['subject'] . '</span>' : '');
			else
				echo '
											<a href="', $attachment['href'], ';image" id="link_', $attachment['id'], '" rel="new_win nofollow" target="_blank"><img class="ila_attach2" style="width: ' . $attachment['width'] . '; height: ' . $attachment['height'] . ';" class="lazyload" src="' . $attachment['href'] . '" alt="" /></a><br />';
		}

		if (!$attachment['is_approved'])
		{
			$clipPNG = $settings['images_url'] . '/icons/clip.png';
			$clipGIF = $settings['images_url'] . '/ila_clip.gif';
			$clipIcon = (file_exists($clipPNG) ? $clipPNG : $clipGIF);

			echo '
											<a href="' . $attachment['href'] . '"><img src="' . $clipIcon . '" style="vertical-align: middle;" alt="*" />&nbsp;' . $attachment['name'] . '</a>';


			if ($context['can_approve'])
				echo '
											&nbsp;[<a href="', $scripturl, '?action=attachapprove;sa=approve;aid=', $attachment['id'], ';', $context['session_var'], '=', $context['session_id'], '">', $txt['approve'], '</a>]&nbsp;|&nbsp;[<a href="', $scripturl, '?action=attachapprove;sa=reject;aid=', $attachment['id'], ';', $context['session_var'], '=', $context['session_id'], '">', $txt['delete'], '</a>]';

			echo '
											(' . $attachment['size'] . ' ~ ' . (!$attachment['is_image'] ? $txt['ila_downloads'] : $txt['ila_views']) . $attachment['downloads'] . ')';

			if (file_exists($sourcedir . '/Mod-WhoDownloadedAttachment.php'))
				call_integration_hook('integrate_attachment_download_list', array(&$attachment));
		}
		else
			echo $download_counter;

		echo ($row_below && $per_row != 1 && $attachment['is_approved'] ? '<br />' : '') . '
										</div>';

		// Next attachment line
		if (!empty($per_row))
			if (++$count % $per_row === 0)
				echo '
										<div class="clear"></div>';
	}

	// If we had unapproved attachments clean up.
	if ($last_approved_state == 0)
		echo '
									</fieldset>';

	echo '
								</div>';
}

//=======================================================================================
// Function for creating buffer for displaying images below the post.
//=======================================================================================
function ILA_Below_Post_Buffer($buffer)
{
	global $context;

	$replacements = array();

	// Need to change the Highslide 'div' elements to <span>' elements
	// so that the following search/replace works.
	if (!empty($context['subject']))
	{
		$search = '<div class="highslide-heading">' . $context['subject'] . '</div>';
		$replace = '<span class="highslide-heading">' . $context['subject'] . '</span>';
		$replacements[$search] = $replace;
		$new_buffer = str_replace(array_keys($replacements), array_values($replacements), $buffer);
	}
	else
		$new_buffer = $buffer;

	$search = '~<div id="msg_(\d+)_footer" class="attachments smalltext">(.*)<\/div>~siU';
	$replace = '<!-- Post and PM Inline Attachments - Display Attachments Below Post --><div>';
	$new_buffer = preg_replace($search, $replace, $new_buffer);

	return ((isset($_REQUEST['xml']) || empty($context['current_topic'])) ? $buffer : $new_buffer);
}

//=======================================================================================
// Function for adding 'Insert Attachment' options to Post and PersonalMessage templates.
//=======================================================================================
function ILA_Post_Template()
{
	global $boarddir, $context, $ila_tag, $modSettings, $sourcedir;

	$ila_tag = (isset($modSettings['ila_insert_tag']) ? $modSettings['ila_insert_tag'] : 'attachment');
	$ila_tag .= (empty($modSettings['ila_insert_format']) ? '=' : ' id=') . '\' + id + \'';
	$ila_tag .= (!empty($modSettings['ila_insert_format']) && $modSettings['ila_insert_format'] == 2 ? ' msg=' . (isset($_REQUEST['msg']) ? $_REQUEST['msg'] : 'new') : '');

	echo '
		<dd class="smalltext">
			<script type="text/javascript"><!-- // --><![CDATA[
				function insertAttachment(id)
				{',
					// Support for 'SCEditor Integration for SMF 2.0.x' mod.
					// https://custom.simplemachines.org/index.php?mod=4319
					(file_exists($boarddir . '/Themes/default/scripts/sceditor.js') && file_exists($boarddir . '/Themes/default/css/sceditorframe.css') ?
							'sceditor.instance(document.forms.postmodify.' . $context['post_box_name'] . ').insertText(\' [' . $ila_tag . '] \');' :
						// Support for 'SMFPacks Advanced Editor' mod.
						// https://www.smfpacks.com/wysiwyg/
						(file_exists($sourcedir . '/AdvancedEditor.php') && !empty($modSettings['advanced_editor_master']) ?
								'sceditor.instance($smfpacksEditor("#' . $context['post_box_name'] . '").get(0)).insertText(\' [' . $ila_tag . '] \');' :
							// Support for discontinued 'SCEditor4SMF' mod.
							// http://custom.simplemachines.org/mods/index.php?mod=3594
							(file_exists($sourcedir . '/Subs-SCEditor.php') ?
									'$(smfmod_editor).data(\'sceditor\').insertText(\' [' . $ila_tag . '] \');' :
								// If no fancy editors use default code for inserting the ILA tag ...
								'replaceText(\' [' . $ila_tag . '] \', document.forms.postmodify.' . $context['post_box_name'] . ');'
							)
						)
					), '
				}
			// ]]></script>
		</dd>';

	return $ila_tag;
}

//=======================================================================================
// Function for getting attachments for Print Page.
//=======================================================================================
function ILA_printpageAttachments($messages)
{
	global $context, $smcFunc;

	// Fetch attachments so we can print/list them if required
	$request = $smcFunc['db_query']('', '
		SELECT a.id_attach, a.id_thumb, a.id_msg, a.filename, a.mime_type, a.width, a.height
		FROM {db_prefix}attachments AS a
		WHERE a.id_msg IN ({array_int:message_list})
			AND a.attachment_type = {int:attachment_type}
			AND a.approved = {int:is_approved}',
		array(
			'message_list' => $messages,
			'attachment_type' => 0,
			'is_approved' => 1,
		)
	);
	$temp = array();
	while ($row = $smcFunc['db_fetch_assoc']($request))
		$temp[$row['id_attach']] = $row;
	$smcFunc['db_free_result']($request);
	ksort($temp);

	foreach ($temp as $row)
		$context['printattach'][$row['id_msg']][] = $row;
}

//=======================================================================================
// Functions for adding ILA code/information to the PrintPage template.
//=======================================================================================
function ILA_printpageAbove()
{
	global $modSettings, $user_info;

	if ($user_info['id'] == 0 || !empty($modSettings['ila_ipp_thumbnail']))
	{
		$inline_width = !empty($modSettings['attachmentThumbWidth']) ? $modSettings['attachmentThumbWidth'] . 'px' : '100%';
		$inline_height = !empty($modSettings['attachmentThumbHeight']) ? $modSettings['attachmentThumbHeight'] . 'px' : '100%';
		$other_width = !empty($modSettings['attachmentThumbWidth']) ? ($modSettings['attachmentThumbWidth'] / 2) . 'px' : '50%';
		$other_height = !empty($modSettings['attachmentThumbHeight']) ? ($modSettings['attachmentThumbHeight'] / 2) . 'px' : '50%';
	}
	else
	{
		$inline_width = $inline_height = $modSettings['ila_ipp_inline_wh'] . 'px';
		$other_width = $other_height = $modSettings['ila_ipp_other_wh'] . 'px';
	}

	echo '
			#footer
			{
				font-family: Verdana, sans-serif;
			}
			@media print
			{
				.print_options
				{
					display: none;
				}
			}
			@media screen
			{
				.print_options
				{
					margin: 1em 0;
				}
			}
			.ila_attach, .bbc_img
			{
				width: auto;
				height: auto;
				max-width: ' . $inline_width . ';
				max-height: ' . $inline_height . ';
				display: ' . (isset($_REQUEST['images']) ? 'inline-block' : 'none') . ';
			}
			img.ila_image_below, .bbc_standard_quote > div > img
			{
				width: auto;
				height: auto;
				max-width: ' . $other_width . ';
				max-height: ' . $other_height . ';
			}
			.ila_link
			{
				display: block;
				margin-left: 1.2em;
			}
			.ila_img
			{
				float: left;
				margin-top: 0.5em;
			}
			dl#posts
			{
				line-height: 1.4em;
			}
			dt.postheader
			{
				border: solid #000;
				border-width: 1px 0;
				padding: 4px 0;
				margin-top: 20px;
			}
			dd.postbody
			{
				margin: 1em 0 2em 2em;
			}
			blockquote
			{
				background-color: whitesmoke;
			}';
}

function ILA_printpageMain()
{
	global $context, $modSettings, $options, $scripturl, $settings, $sourcedir, $topic, $txt;

	$smf20 = (substr($modSettings['smfVersion'], 0, 3) == '2.0');
	$smf21 = (substr($modSettings['smfVersion'], 0, 3) == '2.1');
	$ila_duplicate = !empty($modSettings['ila_duplicate']);
	$row_below = !empty($modSettings['ila_below_post_show']);
	$per_row = ($row_below ? (empty($modSettings['ila_below_post_number']) ? 0 : $modSettings['ila_below_post_number']) : 1);

	// Integration with CustomFieldFilterPost mod
	$customFieldFilterPost = file_exists($sourcedir . '/Subs-CustomFieldFilterPost.php');

	foreach ($context['posts'] as $post)
	{
		// Integration with CustomFieldFilterPost mod
		if ($customFieldFilterPost)
		{
			$context['cffp']['display_readonly'] = $post['cffp']['display_readonly'];
			$context['cffp']['display_member_readonly'] = $post['cffp']['display_member_readonly'];
		}

		echo '
			<dt class="postheader">
				', $txt['title'], ': <strong>', $post['subject'], '</strong><br />
				', $txt['post_by'], ': <strong>', $post['member'], '</strong> ', $txt['search_on'], ' <strong>', $post['time'], '</strong>
			</dt>
			<dd class="postbody">';

		// Integration with CustomFieldFilterPost mod
		if ($customFieldFilterPost)
			template_cffp_show('display_readonly');

		echo '
				', $post['body'];

		// Integration with CustomFieldFilterPost mod
		if ($customFieldFilterPost)
			template_cffp_show('display_member_readonly');

		// Find out which attachments are displayed inline.
		preg_match_all('~0;attach=([0-9]+)?(;image|" type=")~', $post['body'], $inline);

		// Show attachment images below the message body.
		$count = 0;
		$attachmentsBelow = false;
		if (isset($_GET['images']) && !empty($context['printattach'][$post['id_msg']]))
		{
			foreach ($context['printattach'][$post['id_msg']] as $attach)
			{
				if ((($smf20 && ($ila_duplicate && !in_array($attach['id_attach'], $inline[1]) && !in_array($attach['id_thumb'], $inline[1])) || !$ila_duplicate))
				|| ($smf21 && !isset($context['show_attach_under_post'][$attach['id_attach']])))
				{
					if (!$attachmentsBelow)
					{
						$attachmentsBelow = true;
						echo '<p style="padding-top: 0.5em"><hr><hr><strong class="smalltext">' . $txt['ila_attachments_below'] . '</strong></p>';
					}

					$attach_id = !empty($attach['id_thumb']) ? $attach['id_thumb'] : $attach['id_attach'];

					if (!empty($attach['mime_type']) && strpos($attach['mime_type'], 'image/') === 0)
						echo '
							<img class="ila_image_below lazyload" src="', $scripturl . '?action=dlattach;topic=' . $topic . '.0;attach=' . $attach_id . '" alt="">';
					else
						echo '
							<span class="smalltext" style="vertical-align: top;">&nbsp;&nbsp;<strong>' . $attach['filename'] . '</strong>&nbsp;&nbsp;</span>';

					if ($smf20 && (!$row_below || ($per_row && (++$count % $per_row === 0))))
						echo '<p></p>';
				}
			}
		}

		echo '
			</dd>';
	}
}

function ILA_printpageOptions()
{
	global $scripturl, $topic, $txt;

	loadLanguage('InlineAttachments');

	$url_text = $scripturl . '?action=printpage;topic=' . $topic . '.0';
	$url_images = $url_text . ';images';

	echo '
		<div class="print_options">';

	// Which option is set, text or text and attachments
	if (isset($_REQUEST['images']))
		echo '
			<a href="', $url_text, '">', $txt['ila_print_page_text'], '</a> | <strong><a href="', $url_images, '">', $txt['ila_print_page_attachments'], '</a></strong>';
	else
		echo '
			<strong><a href="', $url_text, '">', $txt['ila_print_page_text'], '</a></strong> | <a href="', $url_images, '">', $txt['ila_print_page_attachments'], '</a>';

	echo '
		</div><!-- .print_options -->';
}

?>
