<?php

/***********************************************************************************************************
* Subs-InlineAttachmentsAdmin.php - Subs of Post and PM Inline Attachments mod (v7.08)                     *
************************************************************************************************************
* This mod is licensed under the 2-Clause BSD License, which can be found here:                            *
*	https://opensource.org/licenses/BSD-2-Clause                                                           *
************************************************************************************************************
* CCopyright 2014-2020 Douglas Orend, 2018-2022 Kathy Leslie                                               *
* Redistribution and use in source and binary forms, with or without modification, are permitted provided  *
* that the following conditions are met:                                                                   *
* 1. Redistributions of source code must retain the above copyright notice, this list of conditions        *
*    and the following disclaimer.                                                                         *
* 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and *
*    the following disclaimer in the documentation and/or other materials provided with the distribution.  *
* This software is provided by the copyright holders and contributors "as is" and any express or implied   *
* warranties, including, but not limited to, the implied warranties of merchantability and fitness for a   *
* particular purpose are disclaimed. In no event shall the copyright holder or contributors be liable for  *
* any direct, indirect, incidental, special, exemplary, or consequential damages (including, but not       *
* limited to, procurement of substitute goods or services; loss of use, data, or profits; or business      *
* interruption) however caused and on any theory of liability, whether in contract, strict liability, or   *
* tort (including negligence or otherwise) arising in any way out of the use of this software, even if     *
* advised of the possibility of such damage.                                                               *
***********************************************************************************************************/

if (!defined('SMF'))
	die('Hacking attempt...');

//================================================================================
// Hook functions to add new subsection to the Modifications Setting page.
//================================================================================
function ILA_Admin_Menu_Hook(&$area)
{
	global $txt;
	loadLanguage('InlineAttachmentsAdmin');
	$area['layout']['areas']['manageattachments']['subsections']['ila'] = array($txt['ila_admin_settings']);
}

function ILA_Admin_Settings_Hook(&$sub)
{
	$sub['ila'] = 'ILA_Admin_Settings';
}

function ILA_Admin_Settings($return_config = false)
{
	global $context, $modSettings, $txt, $scripturl, $sourcedir;
	isAllowedTo('admin_forum');

	// Load required stuff in order to make this work right.
	require_once($sourcedir . '/ManagePermissions.php');
	require_once($sourcedir . '/ManageServer.php');
	loadLanguage('InlineAttachments');

	// Make sure that the following settings exist to avoid errors, cause we hate errors!
	$context['settings_insert_above'] = '';
	$smf20 = (substr($modSettings['smfVersion'], 0, 3) == '2.0');

	// If the "done" parameter was passed in the URL, get the count from it and display it.
	if (isset($_GET['done']))
	{
		$completed = (!empty($_SESSION['pms_completed']) ? (int) $_SESSION['pms_completed'] . ' PMs & ' : '') . (int) $_GET['done'];
		$context['settings_insert_above'] .= '<div class="information"><p>' . sprintf($txt['ila_posts_completed'], $completed) . '</p></div>';
	}

	// Assemble the options available in this mod.
	if (!isset($modSettings['ila_insert_tag']))
		$modSettings['ila_insert_tag'] = 'attachment';
	$tags = array();
	foreach (ILA_tags() as $tag)
	{
		// The 'attachimg' tag is only included for backward compatibility for displaying inline attachments
		// that used this particular tag which was only included in some very early versions of the mod.
		// As such, it is not a valid tag option for inserting new images inline.
		if ($tag == 'attachimg')
			continue;

		$tags[$tag] = $tag;
	}

	if ($smf20)
		$config_vars = array(
			array('select', 'ila_insert_tag', $tags),
			array('check', 'ila_attach_same_as_attachment'),
			array('select', 'ila_insert_format', array('[attachment=0]', '[attachment id=0]', '[attachment id=0 msg=new]')),
			'',
			array('int', 'ila_max_width', 'javascript' => 'onchange="validateValue(\'ila_max_width\', 0);"'),
			array('int', 'ila_max_height', 'javascript' => 'onchange="validateValue(\'ila_max_height\', 0);"'),
			array('check', 'ila_enable_responsive'),
			'',
			array('check', 'ila_highslide', ((function_exists('hs4smf') || function_exists('highslide_images') || (!empty($modSettings['enable_jqlightbox_mod']) && strpos($context['html_headers'], 'jquery.prettyPhoto.css'))) ? 99 : 'disabled') => true),
			array('check', 'ila_one_based_numbering', 'javascript' => 'onchange="renumberAttachmentTags();"', 'postinput' => '<input type="hidden" name="renumber_attachment_tags" id="renumber_attachment_tags" value="0" />'),
			array('check', 'ila_no_invalid_attachment_warning'),
			array('check', 'ila_allow_quoted_images'),
			array('check', 'ila_duplicate'),
			array('int', 'ila_transparent', 'javascript' => 'onchange="validateOpacity();"'),
			array('select', 'ila_download_count', array($txt['ila_download_count_n'], $txt['ila_download_count_f'], $txt['ila_download_count_fs'], $txt['ila_download_count_fsd'], $txt['ila_download_count_fsdc'], $txt['ila_download_count_fsdc2'], $txt['ila_download_count_fsdc3'], $txt['ila_download_count_fsd2'], $txt['ila_download_count_fsd3'], $txt['ila_download_count_sc2'])),
			'',
			array('check', 'ila_embed_video_files', 'javascript' => 'onchange="toggleVideo();"'),
			array('callback', 'ila_hidden_video_start', 'type' => 'callback'),	// <== Begin hidden video options section
			array('int', 'ila_video_default_width', 'javascript' => 'onchange="validateValue(\'ila_video_default_width\', 0);"'),
			array('int', 'ila_video_default_height', 'javascript' => 'onchange="validateValue(\'ila_video_default_height\', 0);"'),
			array('check', 'ila_video_show_download_link'),
			array('check', 'ila_video_html5'),
			array('callback', 'ila_hidden_video_end', 'type' => 'callback'),	// <== Finish hidden video options section
			'',
			array('check', 'ila_embed_svg_files', (file_exists($sourcedir . '/Subs-SVG.php') ? 99 : 'disabled') => true),
			'',
			array('check', 'ila_embed_txt_files'),
			array('check', 'ila_embed_pdf_files'),
			'',
			array('check', 'ila_turn_nosniff_off', ($smf20 ? 99 : 'disabled') => true),
			array('check', 'ila_display_exif', (file_exists($sourcedir . '/exif.php') ? 99 : 'disabled') => true),
			'',
			array('check', 'ila_ipp_thumbnail', 'javascript' => 'onchange="toggleVideo();"'),
			array('callback', 'ila_hidden_ipp_start', 'type' => 'callback'),	// <== Begin hidden ipp options section
			array('int', 'ila_ipp_inline_wh', 'javascript' => 'onchange="validateValue(\'ila_ipp_inline_wh\', 150);"'),
			array('int', 'ila_ipp_other_wh', 'javascript' => 'onchange="validateValue(\'ila_ipp_other_wh\', 75);"'),
			array('callback', 'ila_hidden_ipp_end', 'type' => 'callback'),	// <== Finish hidden ipp options section
			'',
			array('check', 'ila_below_post_show', 'javascript' => 'onchange="toggleVideo();"'),
			array('callback', 'ila_hidden_below_start', 'type' => 'callback'),	// <== Begin hidden below options section
			array('int', 'ila_below_post_number', 'javascript' => 'onchange="validateValue(\'ila_below_post_number\', 0);"'),
			array('select', 'ila_below_post_count', array($txt['ila_below_post_count_n'], $txt['ila_below_post_count_f'], $txt['ila_below_post_count_fs'], $txt['ila_below_post_count_fsd'], $txt['ila_below_post_count_fsdc'], $txt['ila_below_post_count_fsdc2'], $txt['ila_below_post_count_fsdc3'], $txt['ila_below_post_count_fsd2'], $txt['ila_download_count_fsd3'], $txt['ila_below_post_count_sc2'])),
			array('callback', 'ila_hidden_below_end', 'type' => 'callback'),	// <== Finish hidden below options section
		);
	else
		$config_vars = array(
			array('int', 'ila_max_width', 'javascript' => 'onchange="validateValue(\'ila_max_width\', 0);"'),
			array('int', 'ila_max_height', 'javascript' => 'onchange="validateValue(\'ila_max_height\', 0);"'),
			array('check', 'ila_enable_responsive'),
			array('ila_line', '.'),
			array('check', 'ila_one_based_numbering', 'javascript' => 'onchange="renumberAttachmentTags();"', 'postinput' => '<input type="hidden" name="renumber_attachment_tags" id="renumber_attachment_tags" value="0" />'),
			array('check', 'ila_no_invalid_attachment_warning'),
			array('check', 'ila_allow_quoted_images'),
			array('int', 'ila_transparent', 'javascript' => 'onchange="validateOpacity();"'),
			array('select', 'ila_download_count', array($txt['ila_download_count_n'], $txt['ila_download_count_f'], $txt['ila_download_count_fs'], $txt['ila_download_count_fsd'], $txt['ila_download_count_fsdc'], $txt['ila_download_count_fsdc2'], $txt['ila_download_count_fsdc3'], $txt['ila_download_count_fsd2'], $txt['ila_download_count_fsd3'], $txt['ila_download_count_sc2'])),
			array('ila_line', '..'),
			array('check', 'ila_embed_video_files', 'javascript' => 'onchange="toggleVideo();"'),
			array('callback', 'ila_hidden_video_start', 'type' => 'callback'),	// <== Begin hidden video options section
			array('int', 'ila_video_default_width', 'javascript' => 'onchange="validateValue(\'ila_video_default_width\', 0);"'),
			array('int', 'ila_video_default_height', 'javascript' => 'onchange="validateValue(\'ila_video_default_height\', 0);"'),
			array('check', 'ila_video_show_download_link'),
			array('check', 'ila_video_html5'),
			array('callback', 'ila_hidden_video_end', 'type' => 'callback'),	// <== Finish hidden video options section
			array('ila_line', '...'),
			array('check', 'ila_embed_svg_files', (file_exists($sourcedir . '/Subs-SVG.php') ? 99 : 'disabled') => true),
			array('ila_line', '....'),
			array('check', 'ila_embed_txt_files'),
			array('check', 'ila_embed_pdf_files'),
			array('ila_line', '.....'),
			array('check', 'ila_ipp_thumbnail', 'javascript' => 'onchange="toggleVideo();"'),
			array('callback', 'ila_hidden_ipp_start', 'type' => 'callback'),	// <== Begin hidden ipp options section
			array('int', 'ila_ipp_inline_wh', 'javascript' => 'onchange="validateValue(\'ila_ipp_inline_wh\', 150);"'),
			array('int', 'ila_ipp_other_wh', 'javascript' => 'onchange="validateValue(\'ila_ipp_other_wh\', 75);"'),
			array('callback', 'ila_hidden_ipp_end', 'type' => 'callback'),	// <== Finish hidden ipp options section
		);

	if ($return_config)
		return $config_vars;

	// Set up for showing settings to the user.
	$context['sub_template'] = 'show_settings';
	$context['settings_title'] = $txt['ila_title'];
	$context['post_url'] = $scripturl . '?action=admin;area=manageattachments;sa=ila;save';

	// We want javascript for our video options.
	if ($smf20)
		$context['settings_insert_below'] = '
			<script type="text/javascript"><!-- // --><![CDATA[
				function toggleVideo()
				{
					// Video display options.
					var checked = document.getElementById("ila_embed_video_files").checked;
					document.getElementById("ila_hidden_video").style.display = (checked ? "" : "none");

					// Images on Print Page display options.
					var checked = document.getElementById("ila_ipp_thumbnail").checked;
					document.getElementById("ila_hidden_ipp").style.display = (checked ? "none" : "");

					// Below Post display options.
					var checked = document.getElementById("ila_below_post_show").checked;
					document.getElementById("ila_hidden_below").style.display = (checked ? "" : "none");
				}
				function validateOpacity()
				{
					var value = document.getElementById("ila_transparent").value;
					document.getElementById("ila_transparent").value = Math.max(0, Math.min(100, value));
				}
				function validateValue(field, min)
				{
					var value = document.getElementById(field).value;
					document.getElementById(field).value = Math.max(min, value);
				}
				function renumberAttachmentTags()
				{
					// V7.03: Changed the \'confirmation\' message to an \'alert\' message because if \'Cancel\' was clicked
					// instead of \'OK\' for the \'confirmation\' message the inline attachment IDs in existing messages
					// were not renumbered resulting in invalid and/or mismatched inline attachment IDs.
					alert("' . $txt['ila_one_based_numbering_alert'] . '");
					document.getElementById("renumber_attachment_tags").value = 1;
				}
				toggleVideo();
			// ]]></script>';
	else
	{
		 addInlineJavaScript('
			function toggleVideo()
			{
				// Video display options.
				var checked = document.getElementById("ila_embed_video_files").checked;
				document.getElementById("ila_hidden_video").style.display = (checked ? "" : "none");

				// Images on Print Page display options.
				var checked = document.getElementById("ila_ipp_thumbnail").checked;
				document.getElementById("ila_hidden_ipp").style.display = (checked ? "none" : "");
			}
			function validateOpacity()
			{
				var value = document.getElementById("ila_transparent").value;
				document.getElementById("ila_transparent").value = Math.max(0, Math.min(100, value));
			}
			function validateValue(field, min)
			{
				var value = document.getElementById(field).value;
				document.getElementById(field).value = Math.max(min, value);
			}
			function renumberAttachmentTags()
			{
				// V7.03: Changed the \'confirmation\' message to an \'alert\' message because if \'Cancel\' was clicked
				// instead of \'OK\' for the \'confirmation\' message the inline attachment IDs in existing messages
				// were not renumbered resulting in invalid and/or mismatched inline attachment IDs.
				alert("' . $txt['ila_one_based_numbering_alert'] . '");
				document.getElementById("renumber_attachment_tags").value = 1;
			}');

		addInlineJavaScript('toggleVideo();', true);
	}

	// Saving?
	if (isset($_GET['save']))
	{
		checkSession();
		$_POST['ila_transparent'] = min(100, max(0, (isset($_POST['ila_transparent']) ? $_POST['ila_transparent'] : 40)));
		$old = (empty($modSettings['ila_one_based_numbering']) ? 0 : 1);
		saveDBSettings($config_vars);
		$redirect = true;
		unset($_SESSION['total_messages'], $_SESSION['pms_completed'], $_SESSION['posts_completed']);
		if (!empty($_POST['renumber_attachment_tags']))
		{
			if (!empty($old) && empty($modSettings['ila_one_based_numbering']))
				$redirect = ILA_Admin_Adjust(false);
			elseif (empty($old) && !empty($modSettings['ila_one_based_numbering']))
				$redirect = ILA_Admin_Adjust(true);

			if ($redirect)
				redirectexit('action=admin;area=manageattachments;sa=ila;done=' . $_SESSION['posts_completed']);
		}
	}
	// Haven't completed the renumbering thing yet?
	elseif (isset($_GET['renumber']))
	{
		checkSession();
		if (ILA_Admin_Adjust( isset($_POST['ascend']) ))
			redirectexit('action=admin;area=manageattachments;sa=ila;done=' . $_SESSION['posts_completed']);
	}
	prepareDBSettingContext($config_vars);
}

//================================================================================
// Template callback functions needed for hiding below post options.
//================================================================================
function template_callback_ila_hidden_below_start()
{
	echo '<div id="ila_hidden_below">';
}

function template_callback_ila_hidden_below_end()
{
	echo '</div>';
}

//================================================================================
// Template callback functions needed for hiding video options.
//================================================================================
function template_callback_ila_hidden_video_start()
{
	echo '<div id="ila_hidden_video">';
}

function template_callback_ila_hidden_video_end()
{
	echo '</div>';
}

//================================================================================
// Template callback functions needed for hiding 'Image on Print Page' options.
//================================================================================
function template_callback_ila_hidden_ipp_start()
{
	echo '<div id="ila_hidden_ipp">';
}

function template_callback_ila_hidden_ipp_end()
{
	echo '</div>';
}

//================================================================================
// Helper function to deal with adjusting all ILA bbcodes when options change.
//================================================================================
function ILA_Admin_Adjust($ascending = true)
{
	global $context, $db_unbuffered, $modSettings, $smcFunc, $txt;

	isAllowedTo('admin_forum');

	// Set up to the context.
	$context['page_title'] = $txt['not_done_title'];
	$context['continue_countdown'] = 3;
	$context['continue_post_data'] = '';
	$context['continue_get_data'] = '';
	$context['sub_template'] = 'not_done';

	// Make sure we have enough time and memory ...
	@set_time_limit(600);
	if (@ini_get('memory_limit') < 256)
		@ini_set('memory_limit', '256M');

	// Set some variables before we start.
	$_REQUEST['start'] = !isset($_REQUEST['start']) ? 0 : (int) $_REQUEST['start'];
	$increment = 250;
	$change_by = ($ascending ? 1 : -1);
	$tags = ILA_tags();

	// Process personal messages and posts containing 'attach' ILA tags.
	$tables = array('personal_messages', 'messages');
	foreach($tables AS $table)
	{
		$id = ($table == 'personal_messages' ? 'id_pm' : 'id_msg');
		$body = ($table == 'personal_messages' ? '%[attach%=%' : '%[attach%=%msg%');

		if (!isset($_SESSION['total_messages']) && (!isset($_SESSION['pms_completed']) || !isset($_SESSION['posts_completed'])))
		{
			// Get the number of PMs/posts containing an ILA tag.
			$request = $smcFunc['db_query']('', '
				SELECT COUNT(*) AS count
				FROM {db_prefix}{raw:table}
				WHERE body LIKE {string:body}',
				array(
					'table' => $table,
					'body' => $body,
				)
			);
			list ($_SESSION['total_messages']) = $smcFunc['db_fetch_row']($request);
			$smcFunc['db_free_result']($request);

			// Keep a record of the number of PMs and posts that are being processed ...
			if ($table == 'personal_messages')
				$_SESSION['pms_completed'] = $_SESSION['total_messages'];
			else
				$_SESSION['posts_completed'] = $_SESSION['total_messages'];
		}

		// Start processing messages containing 'attach' ILA tags.
		$request = $smcFunc['db_query']('', '
			SELECT ' . $id . ' AS id, body
			FROM {db_prefix}{raw:table}
			WHERE body LIKE {string:body}
			ORDER BY {string:sort_by}
			LIMIT {int:start}, {int:number}',
			array(
				'table' => $table,
				'start' => $_REQUEST['start'],
				'number' => $increment,
				'body' => $body,
				'sort_by' => $id,
			)
		);

		$queries = array();
		$total_rows = 0;
		while ($row = $smcFunc['db_fetch_assoc']($request))
		{
			// Process only the post/PMs that has at least one ILA tag within it.
			$total_rows++;
			$begin = $end = '';
			foreach ($tags as $tag)
			{
				// Does this message have the bbcode we are looking for?
				$pattern = '#\[(' . $tag . '(=| id=))(\d+)(|(,| )(.+?))\]#i' . ($context['utf8'] ? 'u' : '');
				if (preg_match_all($pattern, $row['body'], $attachtags, PREG_PATTERN_ORDER))
				{
					// If ascending, sort in reverse order. If descending, sort in normal order.
					// We do this to keep from overwriting the wrong attachment ID during this operation!
					$attachtags = array_unique($attachtags[0]);
					if ($ascending)
						arsort($attachtags);
					else
						asort($attachtags);

					// Change the attachment ID number that the bbcode is using.
					foreach ($attachtags as $attach)
					{
						if (preg_match('#\[' . $tag . '(=| id=)(\d+)(|(,| )(.+?))\]#i', $attach, $params))
						{
							$begin .= 'REPLACE(';
							$end .= ', "[' . $tag . $params[1] . $params[2] . $params[3] . ']", "[' . $tag . $params[1] . ($params[2] + $change_by) . $params[3] . ']")';
						}
					}
				}
			}

			// We can't write anything back to the database using the same database connection otherwise we lose our place in the query.
			// So let's store the change we need to make in an array for later.
			if (!empty($begin))
				$queries[] = 'SET body = ' . $begin . 'body' . $end . ' WHERE ' . $id . ' = ' . ((int) $row['id']);
		}

		// Close the query, then update the database for all posts processed.
		foreach ($queries as $query)
			$smcFunc['db_query']('', 'UPDATE {db_prefix}'. $table . ' ' . $query, array());

		// Continue?
		if ($total_rows == $increment)
		{
			if ($_REQUEST['start'] > $increment)
				$txt['not_done_reason']  = $txt['not_done_reason'] . '<br />(' . ($_REQUEST['start'] - $increment) . $txt['ila_one_based_numbering_total'] . $_SESSION['total_messages'] . $txt['ila_one_based_numbering_batches'] . $increment . ')<br />';

			$_REQUEST['start'] += $increment;
			$context['continue_percent'] = round(100 * $_REQUEST['start'] / $_SESSION['total_messages']);
			$context['continue_get_data'] = '?action=admin;area=manageattachments;sa=ila;renumber;start=' . $_REQUEST['start'];
			$context['continue_post_data'] = '
			<input type="hidden" name="' . $context['session_var'] . '" value="' . $context['session_id'] . '" />
			<input type="hidden" name="' . ($ascending ? 'ascend' : 'descend') . '" value="1" />';


			if (function_exists('apache_reset_timeout'))
				apache_reset_timeout();

			return;
		}
		else
		{
			$smcFunc['db_free_result']($request);

			// Finished processing PMs - now on to the posts ...
 			if (!isset($_SESSION['posts_completed']))
				unset($_SESSION['total_messages']);
		}
	}

	// Adjust the boilerplate strings so they are correct.
	if (isset($modSettings['boiler_content']))
	{
		$setting = $modSettings['boiler_content'];
		foreach ($tags as $tag)
		{
			// Does this string have the bbcode we are looking for?
			$pattern = '#\[' . $tag . '(=| id=)(\d+)(|(,| )(.+?))\]#i' . ($context['utf8'] ? 'u' : '');
			if (!preg_match_all($pattern, $setting, $attachtags, PREG_PATTERN_ORDER))
				continue;

			// If ascending, sort in reverse order. If descending, sort in normal order.
			// We do this to keep from overwriting the wrong attachment ID during this operation!
			$attachtags = array_unique($attachtags[0]);
			if ($ascending)
				arsort($attachtags);
			else
				asort($attachtags);

			// Change the attachment ID number that the bbcode is using.
			foreach ($attachtags as $attach)
			{
				if (preg_match('#\[' . $tag . '(=| id=)(\d+)(|(,| )(.+?))\]#i', $attach, $params))
					$setting = str_replace('[' . $tag . $params[1] . $params[2] . $params[3], '[' . $tag . $params[1] . ($params[2] + $change_by) . $params[3], $setting);
			}
		}

		// Update the database so that the boilerplates are changed accordingly.
		require_once($sourcedir.'/Subs-Admin.php');
		updateSettings(array('boiler_content' => $setting));
	}

	// Signal that we are done adjust the ILA tags.
	return true;
}

?>
