<?php

/***********************************************************************************************************
* db-install.php                                                                                           *
************************************************************************************************************
* This mod is licensed under the 2-Clause BSD License, which can be found here:                            *
*	https://opensource.org/licenses/BSD-2-Clause                                                           *
************************************************************************************************************
* Copyright (c) 2014-2020 Douglas Orend, 2018-2022 Kathy Leslie                                            *
* Redistribution and use in source and binary forms, with or without modification, are permitted provided  *
* that the following conditions are met:                                                                   *
* 1. Redistributions of source code must retain the above copyright notice, this list of conditions        *
*    and the following disclaimer.                                                                         *
* 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and *
*    the following disclaimer in the documentation and/or other materials provided with the distribution.  *
* This software is provided by the copyright holders and contributors "as is" and any express or implied   *
* warranties, including, but not limited to, the implied warranties of merchantability and fitness for a   *
* particular purpose are disclaimed. In no event shall the copyright holder or contributors be liable for  *
* any direct, indirect, incidental, special, exemplary, or consequential damages (including, but not       *
* limited to, procurement of substitute goods or services; loss of use, data, or profits; or business      *
* interruption) however caused and on any theory of liability, whether in contract, strict liability, or   *
* tort (including negligence or otherwise) arising in any way out of the use of this software, even if     *
* advised of the possibility of such damage.                                                               *
************************************************************************************************************
* This file is a simplified database installer. It does what it is supposed to.                            *
***********************************************************************************************************/

$SSI_INSTALL = false;
if (file_exists(dirname(__FILE__) . '/SSI.php') && !defined('SMF'))
{
	$SSI_INSTALL = true;
	require_once(dirname(__FILE__) . '/SSI.php');
}
elseif (!defined('SMF')) // If we are outside SMF and can't find SSI.php, then throw an error
	die('<b>Error:</b> Cannot install - please verify you put this file in the same place as SMF\'s SSI.php.');

require_once($sourcedir.'/Subs-Admin.php');
db_extend('packages');

global $context, $modSettings, $smcFunc;


//=============================================================================================
// Version 7.03 introduced a new download/below_post counter option - if an earlier version
// has been installed change the download/below_post counter from '8' to '9' if necessary.
//=============================================================================================

$currentVersion = !empty($modSettings['ila_version']) ? (float)$modSettings['ila_version'] : '';
if ($currentVersion && $currentVersion < 7.03)
{
	$downloadCount = !empty($modSettings['ila_download_count']) ? $modSettings['ila_download_count'] : 0;
	if ($downloadCount == 8)
		$new['ila_download_count'] = 9;

	$belowPostCount = !empty($modSettings['ila_below_post_count']) ? $modSettings['ila_below_post_count'] : 0;
	if ($belowPostCount == 8)
		$new['ila_below_post_count'] = 9;

	if (!empty($new))
		updateSettings($new);
}


//=============================================================================================
// Save mod version number to database
//=============================================================================================

$contents = file( dirname(__FILE__) . '/package-info.xml' );
if (preg_match('#\<version\>(.+?)\</version\>#i', implode('', $contents), $version))
	$new['ila_version'] = $version[1];
updateSettings($new);


//=============================================================================================
// Disable the SMF WYSIWYG editor for SMF 2.0
//=============================================================================================

$smf20 = (substr($modSettings['smfVersion'], 0, 3) == '2.0');
if ($smf20)
	updateSettings(array('disable_wysiwyg' => true));


//=============================================================================================
// Add options to the settings table
//=============================================================================================

$smcFunc['db_insert']('ignore',
	'{db_prefix}settings',
	array('variable' => 'string', 'value' => 'string'),
	array(
		array('ila_insert_tag', 'attach'),
		array('ila_insert_format', '2'),
		array('ila_max_width', '0'),
		array('ila_max_height', '0'),
		array('ila_enable_responsive', '1'),
		array('ila_highslide', (function_exists('hs4smf') || function_exists('highslide_images') || (!empty($modSettings['enable_jqlightbox_mod']) && strpos($context['html_headers'], 'jquery.prettyPhoto.css'))) ? '1' : '0'),
		array('ila_one_based_numbering', '0'),
		array('ila_no_invalid_attachment_warning', '0'),
		array('ila_allow_quoted_images', '1'),
		array('ila_duplicate', '1'),
		array('ila_transparent', '40'),
		array('ila_video_default_width', '640'),
		array('ila_video_default_height', '400'),
		array('ila_video_html5', '1'),
		array('ila_ipp_thumbnail', '1'),
		array('ila_ipp_inline_wh', '150'),
		array('ila_ipp_other_wh', '75'),
	),
	array('variable')
);


// All finished if this is SMF 2.1 ...
$smf21 = (substr($modSettings['smfVersion'], 0, 3) == '2.1');
if ($smf21)
{
	// Echo that we are done if necessary:
	if ($SSI_INSTALL)
		echo 'Database settings updated ...';
	return;
}


//=============================================================================================
// ILA board permission profiles for SMF 2.0
//=============================================================================================

// Define some variables
$id_group = '-1';
$add_deny = '1';


//=============================================================================================
// Create 'ILA - Prevent Download for Guests - Inline' board permission profile
//=============================================================================================

// Add board permission profile to 'ILA - Prevent Download for Guests - Inline' if it doesn't already exist
$profile_name = 'ILA - Prevent Download for Guests - Inline';

$request = $smcFunc['db_query']('', '
	SELECT COUNT(*)
	FROM {db_prefix}permission_profiles
		WHERE profile_name = {string:profile_name}',
		array('profile_name' => $profile_name)
);
list($profile_exists) = $smcFunc['db_fetch_row']($request);

if (!$profile_exists)
{
	// Create the new board profile
	$smcFunc['db_insert']('',
		'{db_prefix}permission_profiles',
		array('profile_name' => 'string'),
		array($profile_name),
		array('id_profile')
	);
	$profile_id = $smcFunc['db_insert_id']('{db_prefix}permission_profiles', 'id_profile');

	// Copy the permissions from the 'Default' board profile.
	$request = $smcFunc['db_query']('', '
		SELECT id_group, permission, add_deny
		FROM {db_prefix}board_permissions
		WHERE id_profile = {int:copy_from}',
		array('copy_from' => '1')
	);
	$inserts = array();
	while ($row = $smcFunc['db_fetch_assoc']($request))
		$inserts[] = array($profile_id, $row['id_group'], $row['permission'], $row['add_deny']);
	$smcFunc['db_free_result']($request);

	if (!empty($inserts))
		$smcFunc['db_insert']('replace',
			'{db_prefix}board_permissions',
			array('id_profile' => 'int', 'id_group' => 'int', 'permission' => 'string', 'add_deny' => 'int'),
			$inserts,
			array('id_profile', 'id_group', 'permission')
		);

	// Add board permissions to 'ILA - Prevent Download for Guests - Inline' if they don't already exist
	$permission = 'view_attachments';
	$smcFunc['db_insert']('replace',
		'{db_prefix}board_permissions',
		array('id_group' => 'int', 'id_profile' => 'int', 'permission' => 'string', 'add_deny' => 'int'),
		array($id_group, $profile_id, $permission, $add_deny),
		array('id_group', 'id_profile', 'permission', 'add_deny')
	);

	$permission = 'ila_no_download1';
	$smcFunc['db_insert']('replace',
		'{db_prefix}board_permissions',
		array('id_group' => 'int', 'id_profile' => 'int', 'permission' => 'string', 'add_deny' => 'int'),
		array($id_group, $profile_id, $permission, $add_deny),
		array('id_group', 'id_profile', 'permission', 'add_deny')
	);
}


//=============================================================================================
// Create 'ILA - Prevent Download for Guests - Below Post' board permission profile
//=============================================================================================

// Add board permission profile to 'ILA - Prevent Download for Guests - Below Post' if it doesn't already exist
$profile_name = 'ILA - Prevent Download for Guests - Below Post';

$request = $smcFunc['db_query']('', '
	SELECT COUNT(*)
	FROM {db_prefix}permission_profiles
		WHERE profile_name = {string:profile_name}',
		array('profile_name' => $profile_name)
);
list($profile_exists) = $smcFunc['db_fetch_row']($request);

if (!$profile_exists)
{
	// Create the new board profile
	$smcFunc['db_insert']('',
		'{db_prefix}permission_profiles',
		array('profile_name' => 'string'),
		array($profile_name),
		array('id_profile')
	);
	$profile_id = $smcFunc['db_insert_id']('{db_prefix}permission_profiles', 'id_profile');

	// Copy the permissions from the 'Default' board profile.
	$request = $smcFunc['db_query']('', '
		SELECT id_group, permission, add_deny
		FROM {db_prefix}board_permissions
		WHERE id_profile = {int:copy_from}',
		array('copy_from' => '1')
	);
	$inserts = array();
	while ($row = $smcFunc['db_fetch_assoc']($request))
		$inserts[] = array($profile_id, $row['id_group'], $row['permission'], $row['add_deny']);
	$smcFunc['db_free_result']($request);

	if (!empty($inserts))
		$smcFunc['db_insert']('replace',
			'{db_prefix}board_permissions',
			array('id_profile' => 'int', 'id_group' => 'int', 'permission' => 'string', 'add_deny' => 'int'),
			$inserts,
			array('id_profile', 'id_group', 'permission')
		);

	// Add board permission to 'ILA - Prevent Download for Guests - Below Post' if they don't already exist
	$permission = 'view_attachments';
	$smcFunc['db_insert']('replace',
		'{db_prefix}board_permissions',
		array('id_group' => 'int', 'id_profile' => 'int', 'permission' => 'string', 'add_deny' => 'int'),
		array($id_group, $profile_id, $permission, $add_deny),
		array('id_group', 'id_profile', 'permission', 'add_deny')
	);

	$permission = 'ila_no_download2';
	$smcFunc['db_insert']('replace',
		'{db_prefix}board_permissions',
		array('id_group' => 'int', 'id_profile' => 'int', 'permission' => 'string', 'add_deny' => 'int'),
		array($id_group, $profile_id, $permission, $add_deny),
		array('id_group', 'id_profile', 'permission', 'add_deny')
	);
}


//=============================================================================================
// Create 'ILA - Prevent Download for Guests - Inline and Below Post' board permission profile
//=============================================================================================

// Add board permission profile to 'ILA - Prevent Download for Guests - Inline and Below Post' if it doesn't already exist
$profile_name = 'ILA - Prevent Download for Guests - Inline and Below Post';

$request = $smcFunc['db_query']('', '
	SELECT COUNT(*)
	FROM {db_prefix}permission_profiles
		WHERE profile_name = {string:profile_name}',
		array('profile_name' => $profile_name)
);
list($profile_exists) = $smcFunc['db_fetch_row']($request);

if (!$profile_exists)
{
	// Create the new board profile
	$smcFunc['db_insert']('',
		'{db_prefix}permission_profiles',
		array('profile_name' => 'string'),
		array($profile_name),
		array('id_profile')
	);
	$profile_id = $smcFunc['db_insert_id']('{db_prefix}permission_profiles', 'id_profile');

	// Copy the permissions from the 'Default' board profile.
	$request = $smcFunc['db_query']('', '
		SELECT id_group, permission, add_deny
		FROM {db_prefix}board_permissions
		WHERE id_profile = {int:copy_from}',
		array('copy_from' => '1')
	);
	$inserts = array();
	while ($row = $smcFunc['db_fetch_assoc']($request))
		$inserts[] = array($profile_id, $row['id_group'], $row['permission'], $row['add_deny']);
	$smcFunc['db_free_result']($request);

	if (!empty($inserts))
		$smcFunc['db_insert']('replace',
			'{db_prefix}board_permissions',
			array('id_profile' => 'int', 'id_group' => 'int', 'permission' => 'string', 'add_deny' => 'int'),
			$inserts,
			array('id_profile', 'id_group', 'permission')
		);

	// Add board permissions to 'ILA - Prevent Download for Guests - Inline and Below Post' if they don't already exist
	$permission = 'view_attachments';
	$smcFunc['db_insert']('replace',
		'{db_prefix}board_permissions',
		array('id_group' => 'int', 'id_profile' => 'int', 'permission' => 'string', 'add_deny' => 'int'),
		array($id_group, $profile_id, $permission, $add_deny),
		array('id_group', 'id_profile', 'permission', 'add_deny')
	);

	$permission = 'ila_no_download1';
	$smcFunc['db_insert']('replace',
		'{db_prefix}board_permissions',
		array('id_group' => 'int', 'id_profile' => 'int', 'permission' => 'string', 'add_deny' => 'int'),
		array($id_group, $profile_id, $permission, $add_deny),
		array('id_group', 'id_profile', 'permission', 'add_deny')
	);

	$permission = 'ila_no_download2';
	$smcFunc['db_insert']('replace',
		'{db_prefix}board_permissions',
		array('id_group' => 'int', 'id_profile' => 'int', 'permission' => 'string', 'add_deny' => 'int'),
		array($id_group, $profile_id, $permission, $add_deny),
		array('id_group', 'id_profile', 'permission', 'add_deny')
	);
}

// Echo that we are done if necessary:
if ($SSI_INSTALL)
	echo 'Database settings and permission profiles updated ...';
?>
